#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
from numpy import eye, linalg, arange, matrix
import unittest
from types import *
from helpers import *
import numpy
from itertools import *

class Matrixtests(casadiTestCase):
  def test_constructorlol(self):
    self.message("List of list constructor")
    a=DM(array([[1,2,3],[4,5,6],[7,8,9]]))
    b=DM([[1,2,3],[4,5,6],[7,8,9]])
    self.checkarray(a,b,"List of list constructor")

  def test_sum(self):
    self.message("sum")
    D=DM([[1,2,3],[4,5,6],[7,8,9]])
    self.checkarray(c.sum1(D),array([[12,15,18]]),'sum()')
    self.checkarray(c.sum2(D),array([[6,15,24]]).T,'sum()')


  def test_inv(self):
    self.message("Matrix inverse")
    a = DM([[1,2],[1,3]])
    self.checkarray(mtimes(c.inv(a),a),eye(2),"DM inverse")

  def test_trans(self):
    self.message("trans")
    a = DM(0,1)
    b = a.T
    self.assertEqual(b.size1(),1)
    self.assertEqual(b.size2(),0)

  def test_vertcat(self):
    self.message("vertcat")
    A = DM.ones(2,3)
    B = DM(4,3)
    C = vertcat(A,B)

    self.checkarray(C.shape,(6,3),"vertcat shape")
    self.assertEqual(C.nnz(),A.nnz(),"vertcat size")

    self.assertRaises(RuntimeError,lambda : horzcat(A,B))

  def test_horzcat(self):
    self.message("horcat")
    A = DM.ones(3,2)
    B = DM(3,4)
    C = horzcat(A,B)

    self.checkarray(C.shape,(3,6),"horzcat shape")
    self.assertEqual(C.nnz(),A.nnz(),"vertcat size")

    self.assertRaises(RuntimeError,lambda : vertcat(A,B))

  def test_diagcat(self):

    x = MX.sym("x",2,2)
    y = MX.sym("y",Sparsity.lower(3))
    z = MX.sym("z",4,2)

    L = [x,y,z]

    fMX = Function("fMX", L,[diagcat(*L)])

    LSX = [ SX.sym("",i.sparsity()) for i in L ]
    fSX = Function("fSX", LSX,[diagcat(*LSX)])

    for f in [fMX,fSX]:
      for i in range(3):
        f_in[i]=list(range(f.nnz_in(i)))

    self.checkfunction(fMX,fSX)

  def test_veccat(self):
    self.message("vecccat")
    A = DM(2,3)
    A[0,1] = 2
    A[1,0] = 1
    A[1,2] = 3
    B = DM(3,1)
    B[0,0] = 4
    B[1,0] = 5
    B[2,0] = 6
    C = veccat(*[A,B])

    self.checkarray(C.shape,(9,1),"veccat shape")
    self.assertEqual(C.nnz(),A.nnz()+B.nnz(),"veccat size")

    self.checkarray(tuple(C.nonzeros()),tuple(arange(1,7)),"numbers shape")

  def test_slicestepnegative(self):
    self.message("Slice step negative")
    a1 = [1,2,3,4,5]
    a2 = DM(a1)

    self.checkarray(a2[0:4:-1,0],DM(a1[0:4:-1])) # gives empty set
    self.checkarray(a2[4:0:-1,0],DM(a1[4:0:-1])) # gives [5, 4, 3, 2]
    self.checkarray(a2[0:4:-2,0],DM(a1[0:4:-2])) # gives empty set
    self.checkarray(a2[4:0:-2,0],DM(a1[4:0:-2])) # gives [5, 4, 3, 2]
    self.checkarray(a2[1:4:-2,0],DM(a1[1:4:-2])) # gives empty set
    self.checkarray(a2[4:1:-2,0],DM(a1[4:1:-2])) # gives [5, 4, 3, 2]
    self.checkarray(a2[0:3:-2,0],DM(a1[0:3:-2])) # gives empty set
    self.checkarray(a2[3:0:-2,0],DM(a1[3:0:-2])) # gives [5, 4, 3, 2]
    self.checkarray(a2[::-1,0],DM(a1[::-1])) # gives [5, 4, 3, 2, 1]
    self.checkarray(a2[::1,0],DM(a1[::1])) # gives [1,2,3,4,5]
    self.checkarray(a2[2::-1,0],DM(a1[2::-1])) # gives [3,2,1]
    self.checkarray(a2[:2:-1,0],DM(a1[:2:-1])) # gives [5,4]
    self.checkarray(a2[-1::-1,0],DM(a1[-1::-1])) # gives [5,4,3,2,1]
    self.checkarray(a2[:-1:-1,0],DM(a1[:-1:-1])) # gives []

  def test_indexingOutOfBounds(self):
    self.message("Indexing out of bounds")
    y = DM.zeros(4, 5)
    self.assertRaises(RuntimeError,lambda : y[12,0] )
    self.assertRaises(RuntimeError,lambda : y[12,12] )
    self.assertRaises(RuntimeError,lambda : y[0,12] )
    self.assertRaises(RuntimeError,lambda : y[12,:] )
    self.assertRaises(RuntimeError,lambda : y[12:15,0] )
    self.assertRaises(RuntimeError,lambda : y[:,12] )
    self.assertRaises(RuntimeError,lambda : y[0,12:15] )
    y[-1,2]
    self.assertRaises(RuntimeError,lambda : y[-12,2] )
    y[-3:-1,2]
    self.assertRaises(RuntimeError,lambda : y[-12:-9,2] )

    def test():
      y[12,0] = 0
    self.assertRaises(RuntimeError,test)
    def test():
      y[12,12] = 0
    self.assertRaises(RuntimeError,test)
    def test():
      y[0,12] = 0
    self.assertRaises(RuntimeError,test)
    def test():
      y[12,:] = 0
    self.assertRaises(RuntimeError,test)
    def test():
      y[12:15,0] = 0
    self.assertRaises(RuntimeError,test)
    def test():
      y[:,12] = 0
    self.assertRaises(RuntimeError,test)
    def test():
      y[0,12:15] = 0
    self.assertRaises(RuntimeError,test)
    y[-1,2] = 0
    def test():
      y[-12,2] = 0
    self.assertRaises(RuntimeError,test)
    y[-3:-1,2] = 0
    def test():
      y[-12:-9,2] = 0
    self.assertRaises(RuntimeError,test)

  def huge_slice(self):
    self.message("huge slice")
    a = SX.sym("a",Sparsity.diag(50000))

    a[:,:]

  def test_nonmonotonous_indexing(self):
    self.message("non-monotonous indexing")
    # Regression test for #354
    A = DM([[1,2,3],[4,5,6],[7,8,9]])
    B = A[[0,2,1],0]
    self.checkarray(DM([1,7,4]),B,"non-monotonous")

    B = A[0,[0,2,1]]
    self.checkarray(DM([1,3,2]).T,B,"non-monotonous")

  def test_IM_indexing(self):
    self.message("IM")
    A = DM(2,2)
    A[0,0] = 1
    A[1,1] = 3
    A[0,1] = 2
    A[1,0] = 4


    B = DM([1,2,3,4,5])

    B_ = B[A]

    self.checkarray(B_,DM([[2,3],[5,4]]),"matrix indexing")

    B[A] = DM([[1,2],[3,4]])

    self.checkarray(B,DM([1,1,2,4,3]),"matrix indexing assignement")

    #B[A].set(DM([[10,20],[30,40]]))

    #self.checkarray(B,DM([1,10,20,40,30]),"Imatrix indexing setting")

    B = DM([1,2,3,4,5])

    B_ = B[A]

    self.checkarray(array(B_),DM([[2,3],[5,4]]),"Imatrix indexing")

    B[A] = DM([[1,2],[3,4]])

    self.checkarray(array(B),DM([1,1,2,4,3]),"Imatrix indexing assignement")

    B = DM(5,1)

    self.assertRaises(Exception, lambda : B.nz[A])

  def test_sparsity_indexing(self):
    self.message("sparsity")

    B = DM([[1,2,3,4,5],[6,7,8,9,10]])

    A = DM([[1,1,0,0,0],[0,0,1,0,0]])
    A = sparsify(A)
    sp = A.sparsity()


    self.checkarray(B[sp],DM([[1,2,0,0,0],[0,0,8,0,0]]),"sparsity indexing")

    B[sp] = -4

    self.checkarray(B,DM([[-4,-4,3,4,5],[6,7,-4,9,10]]),"sparsity indexing assignement")

    B = DM([[1,2,3,4,5],[6,7,8,9,10]])

    B[sp] = 2*B

    self.checkarray(B,DM([[2,4,3,4,5],[6,7,16,9,10]]),"Imatrix indexing assignement")

    self.assertRaises(Exception, lambda : B[Sparsity.dense(4,4)])

  def test_index_setting(self):
    self.message("index setting")
    B = DM([1,2,3,4,5])

    B[0] = 8
    self.checkarray(B,DM([8,2,3,4,5]),"index setting")
    B[1,0] = 4
    self.checkarray(B,DM([8,4,3,4,5]),"index setting")
    B[:,0] = 7
    self.checkarray(B,DM([7,7,7,7,7]),"index setting")
    #B[0].set(3)
    #self.checkarray(B,DM([3,7,7,7,7]),"index setting")
    #B[0].setAll(4)
    #self.checkarray(B,DM([4,4,4,4,4]),"index setting")


  def test_issue298(self):
    self.message("Issue #298")
    a = DM(4,1)
    b = c.reshape(a,2,2)
    self.assertEqual(type(a),type(b))

  def test_det(self):
    self.message("Determinant")
    npy_det = numpy.linalg.det

    a = DM(1,1)
    a[0,0] = 5
    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(5):
      a[i,i] = i+1

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(4):
      a[i,i] = i+1
    a[0,4] = 3
    a[4,0] = 7

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(5):
      for j in range(5):
        a[i,j] = i+j

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(4):
      for j in range(5):
        a[i,j] = i+j

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(5):
      for j in range(4):
        a[i,j] = i+j

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(4):
      for j in range(5):
        a[i,j] = i+j
    a[4,1] = 12

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(5):
      for j in range(4):
        a[i,j] = i+j
    a[1,4] = 12

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(4):
      for j in range(5):
        a[i,j] = i+j
    a[4,2] = 12

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(5,5)
    for i in range(5):
      for j in range(4):
        a[i,j] = i+j
    a[2,4] = 12

    self.checkarray(det(a),npy_det(a),"det()")

    a = DM(50,50)
    for i in range(50):
      a[i,i] = i+1

    self.checkarray(det(a)/npy_det(a),1,"det()")

  @skip(not GlobalOptions.getSimplificationOnTheFly())
  def test_inv_sparsity(self):
    self.message("sparsity pattern of inverse")

    n = 8

    sp = Sparsity.lower(n)

    x  = SX(sp,vertcat(*[SX.sym("a%d" % i) for i in range(sp.nnz())]))


    x_ = DM.ones(x.sparsity())

    I_ = DM.ones(inv(x).sparsity())

    # For a reducible matrix, struct(A^(-1)) = struct(A)
    self.checkarray(x_,I_,"inv")

    sp = Sparsity.lower(n)

    x = SX.sym("a", sp)
    x[0,n-1] = 1


    I_ = DM.ones(inv(x).sparsity())

    # An irreducible matrix has a dense inverse in general
    self.checkarray(DM.ones(n,n),I_,"inv")

    x = SX.sym("a", sp)
    x[0,int(n/2)] = 1

    s_ = DM.ones(sp)
    s_[:,:int(n/2)+1] = 1

    I_ = DM.ones(inv_minor(x).sparsity())

    s_ = densify(s_)
    T_ = densify(I_)
    # An irreducible matrix does not have to be dense per se
    self.checkarray(s_,I_,"inv")

  def test_mtimes(self):
    A = DM.ones((4,3))
    B = DM.ones((3,8))
    C = DM.ones((8,7))

    self.assertRaises(RuntimeError,lambda : mtimes([]))

    D = mtimes([A])

    self.assertEqual(D.shape[0],4)
    self.assertEqual(D.shape[1],3)

    D = mtimes([A,B])

    self.assertEqual(D.shape[0],4)
    self.assertEqual(D.shape[1],8)

    D = mtimes([A,B,C])

    self.assertEqual(D.shape[0],4)
    self.assertEqual(D.shape[1],7)

  def test_remove(self):
    self.message("remove")
    B = DM([[1,2,3,4],[5,6,7,8],[9,10,11,12],[13,14,15,16],[17,18,19,20]])

    A = DM(B)
    A.remove([],[])
    self.checkarray(A, B,"remove nothing")

    A = DM(B)
    A.remove([],[1])
    self.checkarray(A, DM([[1,3,4],[5,7,8],[9,11,12],[13,15,16],[17,19,20]]),"remove a column")

    A = DM(B)
    A.remove([0,3],[1])
    self.checkarray(A, DM([[5,7,8],[9,11,12],[17,19,20]]),"remove a column and two rows ")

  def test_comparisons(self):
    m = DM
    A = m([[5,4],[2,1]])

    for c in [6,6.0,DM([6]),np.array([6]),matrix(6)]:
      self.checkarray(A<=c,m([[1,1],[1,1]]),"<=")
      self.checkarray(A<c,m([[1,1],[1,1]]),"<")
      self.checkarray(A>c,m([[0,0],[0,0]]),">")
      self.checkarray(A>=c,m([[0,0],[0,0]]),">=")
      self.checkarray(A==c,m([[0,0],[0,0]]),"==")
      self.checkarray(A!=c,m([[1,1],[1,1]]),"!=")

      self.checkarray(c>=A,m([[1,1],[1,1]]),"<=")
      self.checkarray(c>A,m([[1,1],[1,1]]),"<")
      self.checkarray(c<A,m([[0,0],[0,0]]),">")
      self.checkarray(c<=A,m([[0,0],[0,0]]),">=")
      if args.known_bugs or not isinstance(c,matrix):
        self.checkarray(c==A,m([[0,0],[0,0]]),"==")
        self.checkarray(c!=A,m([[1,1],[1,1]]),"!=")

    for c in [5,5.0,DM([5]),np.array([5])]+([matrix(5)] if check_matrix else []):
      self.checkarray(A<=c,m([[1,1],[1,1]]),"<=")
      self.checkarray(A<c,m([[0,1],[1,1]]),"<")
      self.checkarray(A>c,m([[0,0],[0,0]]),">")
      self.checkarray(A>=c,m([[1,0],[0,0]]),">=")
      self.checkarray(A==c,m([[1,0],[0,0]]),"==")
      self.checkarray(A!=c,m([[0,1],[1,1]]),"!=")

      self.checkarray(c>=A,m([[1,1],[1,1]]),"<=")
      self.checkarray(c>A,m([[0,1],[1,1]]),"<")
      self.checkarray(c<A,m([[0,0],[0,0]]),">")
      self.checkarray(c<=A,m([[1,0],[0,0]]),">=")
      if args.known_bugs or not isinstance(c,matrix):
        self.checkarray(c==A,m([[1,0],[0,0]]),"==")
        self.checkarray(c!=A,m([[0,1],[1,1]]),"!=")

    for c in [4,4.0,DM([4]),np.array([4]),matrix(4)]:
      self.checkarray(A<=c,m([[0,1],[1,1]]),"<=")
      self.checkarray(A<c,m([[0,0],[1,1]]),"<")
      self.checkarray(A>c,m([[1,0],[0,0]]),">")
      self.checkarray(A>=c,m([[1,1],[0,0]]),">=")
      if args.known_bugs or not isinstance(c,matrix):
        self.checkarray(A==c,m([[0,1],[0,0]]),"==")
        self.checkarray(A!=c,m([[1,0],[1,1]]),"!=")

      self.checkarray(c>=A,m([[0,1],[1,1]]),"<=")
      self.checkarray(c>A,m([[0,0],[1,1]]),"<")
      self.checkarray(c<A,m([[1,0],[0,0]]),">")
      self.checkarray(c<=A,m([[1,1],[0,0]]),">=")
      if args.known_bugs or not isinstance(c,matrix):
        self.checkarray(c==A,m([[0,1],[0,0]]),"==")
        self.checkarray(c!=A,m([[1,0],[1,1]]),"!=")

    for c in [1,1.0,DM([1]),np.array([1]),matrix(1)]:
      self.checkarray(A<=c,m([[0,0],[0,1]]),"<=")
      self.checkarray(A<c,m([[0,0],[0,0]]),"<")
      self.checkarray(A>c,m([[1,1],[1,0]]),">")
      self.checkarray(A>=c,m([[1,1],[1,1]]),">=")
      self.checkarray(A==c,m([[0,0],[0,1]]),"==")
      self.checkarray(A!=c,m([[1,1],[1,0]]),"!=")

      self.checkarray(c>=A,m([[0,0],[0,1]]),"<=")
      self.checkarray(c>A,m([[0,0],[0,0]]),"<")
      self.checkarray(c<A,m([[1,1],[1,0]]),">")
      self.checkarray(c<=A,m([[1,1],[1,1]]),">=")
      if args.known_bugs or not isinstance(c,matrix):
        self.checkarray(c==A,m([[0,0],[0,1]]),"==")
        self.checkarray(c!=A,m([[1,1],[1,0]]),"!=")

    for c in [0,DM([0]),np.array([0]),matrix(0)]:
      self.checkarray(A<=c,m([[0,0],[0,0]]),"<=")
      self.checkarray(A<c,m([[0,0],[0,0]]),"<")
      self.checkarray(A>c,m([[1,1],[1,1]]),">")
      self.checkarray(A>=c,m([[1,1],[1,1]]),">=")
      self.checkarray(A==c,m([[0,0],[0,0]]),"==")
      self.checkarray(A!=c,m([[1,1],[1,1]]),"!=")

      self.checkarray(c>=A,m([[0,0],[0,0]]),"<=")
      self.checkarray(c>A,m([[0,0],[0,0]]),"<")
      self.checkarray(c<A,m([[1,1],[1,1]]),">")
      self.checkarray(c<=A,m([[1,1],[1,1]]),">=")
      if args.known_bugs or not isinstance(c,matrix):
        self.checkarray(c==A,m([[0,0],[0,0]]),"==")
        self.checkarray(c!=A,m([[1,1],[1,1]]),"!=")

  def test_truth(self):
    self.assertTrue(bool(DM([1])))
    self.assertFalse(bool(DM([0])))
    self.assertTrue(bool(DM([0.2])))
    self.assertTrue(bool(DM([-0.2])))
    self.assertRaises(Exception, lambda : bool(DM([2.0,3])))
    self.assertRaises(Exception, lambda : bool(DM()))

  def test_listslice(self):
    def check(d,rowbase,colbase):
      for col in permutations(colbase):
        for row in permutations(rowbase):
          r = DM.zeros(len(row),len(col))
          for i,ii in enumerate(row):
            for j,jj in enumerate(col):
              r[i,j] = d[ii,jj]
          self.checkarray(d[row,col],r,"%s[%s,%s]" % (repr(d),str(row),str(col)))


    # get1
    check(DM(Sparsity.dense(3,3),list(range(3*3))),[0,1,2],[0,1,2])
    check(DM(Sparsity.dense(4,4),list(range(4*4))),[0,1,3],[0,2,3])
    check(DM(Sparsity.dense(3,3),list(range(3*3))),[0,0,1],[0,0,1])
    check(DM(Sparsity.dense(3,3),list(range(3*3))),[0,0,2],[0,0,2])
    check(DM(Sparsity.dense(3,3),list(range(3*3))),[1,1,2],[1,1,2])

    sp = Sparsity.lower(4)
    d = DM(sp,list(range(sp.nnz())))
    check(d,[0,1,3],[0,2,3])
    check(d.T,[0,1,3],[0,2,3])

    sp = Sparsity.rowcol([0,1,2],[0,1],4,4)
    d = DM(sp,list(range(sp.nnz())))
    check(d,[0,3],[0,2])

    # get2
    check(DM(Sparsity.dense(2,2),list(range(2*2))),[0,0,0],[0,0,0])
    check(DM(Sparsity.dense(2,2),list(range(2*2))),[0,0,1],[0,0,1])
    check(DM(Sparsity.dense(2,2),list(range(2*2))),[1,1,0],[1,1,0])
    check(DM(Sparsity.dense(2,2),list(range(2*2))),[1,1,1],[1,1,1])

    sp = Sparsity.lower(3)
    d = DM(sp,list(range(sp.nnz())))
    check(d,[0,1,2],[0,1,2])
    check(d.T,[0,1,2],[0,1,2])

    sp = Sparsity.rowcol([0,2],[0,1],4,4)
    d = DM(sp,list(range(sp.nnz())))
    check(d,[0,1,3],[0,2,3])

  def test_sparsesym(self):
    # feature removed in 73f407e
    return
    self.message("sparsesym")
    D = DM([[1,2,-3],[2,-1,0],[-3,0,5]])
    D = sparsify(D)

    i = D.getSym()
    #self.checkarray(list(i),[1,2,-1,-3,5])
    A = 2*D
    A.setSym(i)
    self.checkarray(A,D)

  def test_diagcat(self):
    self.message("diagcat")
    C = diagcat(*[DM([[-1.4,-3.2],[-3.2,-28]]),DM([[15,-12,2.1],[-12,16,-3.8],[2.1,-3.8,15]]),1.8,-4.0])
    r = DM([[-1.4,-3.2,0,0,0,0,0],[-3.2,-28,0,0,0,0,0],[0,0,15,-12,2.1,0,0],[0,0,-12,16,-3.8,0,0],[0,0,2.1,-3.8,15,0,0],[0,0,0,0,0,1.8,0],[0,0,0,0,0,0,-4]])
    r = sparsify(r)
    self.checkarray(C,r)

  def test_diag_sparse(self):
    self.message("diag sparse")

    for n in [[0,1,0,0,2,3,4,5,6,0],[1,2,3,0],[0,1,2,3]]:
      d = DM(n)
      D = DM(n)
      d = sparsify(d)
      m = c.diag(d)
      M = sparsify(c.diag(D))

      self.checkarray(m.sparsity().colind(),M.sparsity().colind())
      self.checkarray(m.sparsity().row(),M.sparsity().row())

  def test_sprank(self):
    self.message("sprank")

    a = DM([[1,0,0],[0,1,0],[0,0,1]])
    a = sparsify(a)
    self.assertEqual(sprank(a),3)

    a = DM([[1,0,0],[0,0,0],[0,0,1]])
    a = sparsify(a)
    self.assertEqual(sprank(a),2)

    a = DM([[0,0,0],[0,0,0],[0,0,1]])
    a = sparsify(a)
    self.assertEqual(sprank(a),1)

    a = DM([[0,0,0],[0,0,0],[0,0,0]])
    a = sparsify(a)
    self.assertEqual(sprank(a),0)

    self.assertEqual(sprank(DM.ones(1,3)),1)
    self.assertEqual(sprank(DM.ones(3,1)),1)
    self.assertEqual(sprank(DM.ones(2,3)),2)
    self.assertEqual(sprank(DM.ones(3,2)),2)
    self.assertEqual(sprank(DM.ones(3,3)),3)
    self.assertEqual(sprank(DM.ones(3,3)),3)

    A = DM(6,4)
    A[0,0] = 1
    A[1,2] = 1
    A[2,2] = 1
    A[5,3] = 1

    self.assertEqual(sprank(A),3)

  def test_cross(self):
    self.message("cross products")

    crossc = c.cross

    self.checkarray(crossc(DM([1,0,0]),DM([0,1,0])),DM([0,0,1]))

    self.checkarray(crossc(DM([1.1,1.3,1.7]),DM([2,3,13])),DM([11.8,-10.9,0.7]))
    self.checkarray(crossc(DM([1.1,1.3,1.7]).T,DM([2,3,13]).T),DM([11.8,-10.9,0.7]).T)

    self.checkarray(crossc(DM([[1.1,1.3,1.7],[1,0,0],[0,0,1],[4,5,6]]),DM([[2,3,13],[0,1,0],[0,0,1],[1,0,1]])),DM([[11.8,-10.9,0.7],[0,0,1],[0,0,0],[5,2,-5]]))
    self.checkarray(crossc(DM([[1.1,1.3,1.7],[1,0,0],[0,0,1],[4,5,6]]).T,DM([[2,3,13],[0,1,0],[0,0,1],[1,0,1]]).T),DM([[11.8,-10.9,0.7],[0,0,1],[0,0,0],[5,2,-5]]).T)

    self.checkarray(crossc(DM([[1.1,1.3,1.7],[1,0,0],[0,0,1],[4,5,6]]),DM([[2,3,13],[0,1,0],[0,0,1],[1,0,1]]),2),DM([[11.8,-10.9,0.7],[0,0,1],[0,0,0],[5,2,-5]]))

    self.checkarray(crossc(DM([[1.1,1.3,1.7],[1,0,0],[0,0,1],[4,5,6]]).T,DM([[2,3,13],[0,1,0],[0,0,1],[1,0,1]]).T,1),DM([[11.8,-10.9,0.7],[0,0,1],[0,0,0],[5,2,-5]]).T)

  def test_is_regular(self):
    self.assertTrue(DM([1,2]).is_regular())
    self.assertFalse(DM([1,inf]).is_regular())
    self.assertFalse(DM.nan(2).is_regular())

  def test_sizes(self):
    self.assertEqual(Sparsity.diag(10).nnz_diag(),10)
    self.assertEqual(Sparsity.diag(10).nnz_upper(),10)
    self.assertEqual(Sparsity.diag(10).nnz_lower(),10)
    self.assertEqual(Sparsity.dense(10,10).nnz_lower(),10*11/2)
    self.assertEqual(Sparsity.dense(10,10).nnz_upper(),10*11/2)
    self.assertEqual(Sparsity.dense(10,10).nnz_diag(),10)

    self.assertEqual(sparsify(DM([[1,1,0],[1,0,1],[0,0,0]])).nnz_diag(),1)
    self.assertEqual(sparsify(DM([[1,1,0],[1,0,1],[0,0,0]])).nnz_lower(),2)
    self.assertEqual(sparsify(DM([[1,1,0],[1,0,1],[0,0,0]])).nnz_upper(),3)

  def test_tril2symm(self):
    a = DM(Sparsity.upper(3),list(range(Sparsity.upper(3).nnz()))).T
    s = tril2symm(a)
    self.checkarray(s,DM([[0,1,3],[1,2,4],[3,4,5]]))

    with self.assertRaises(Exception):
      tril2symm(DM.ones(5,3))

    print(DM.ones(5,5).nnz_upper()-DM.ones(5,5).nnz_diag())

    with self.assertRaises(Exception):
      tril2symm(DM.ones(5,5))

  def test_not_null(self):
    x = MX.sym('x',3,1)
    sp = Sparsity.upper(2)
    MX(sp,x)

  def test_segfault(self):
    x = MX.sym('x',10,1)
    sp = Sparsity.upper(2)
    y = triu2symm(MX(sp,x[1:4]))
    f = Function("f", [x],[y])

  def test_append_empty(self):
    a = vertcat(DM(0,0),DM(0,2))

    self.assertEqual(a.size1(),0)
    self.assertEqual(a.size2(),2)

    a = vertcat(DM(0,0),DM(2,0),DM(3,0))

    self.assertEqual(a.size1(),5)
    self.assertEqual(a.size2(),0)

  def test_vertcat_empty(self):
    a = DM(0,2)
    v = vertcat(a,a)

    self.assertEqual(v.size1(),0)
    self.assertEqual(v.size2(),2)

    a = DM(2,0)
    v = vertcat(a,a)

    self.assertEqual(v.size1(),4)
    self.assertEqual(v.size2(),0)

  def test_vertsplit(self):
    a = DM(Sparsity.upper(5),list(range(int(5*6/2)))).T
    v = vertsplit(a,[0,2,4,5])

    self.assertEqual(len(v),3)
    self.checkarray(v[0],DM([[0,0,0,0,0],[1,2,0,0,0]]))
    self.checkarray(v[1],DM([[3,4,5,0,0],[6,7,8,9,0]]))
    self.checkarray(v[2],DM([[10,11,12,13,14]]))

    v = vertsplit(a)
    self.assertEqual(len(v),a.size1())
    self.checkarray(v[0],DM([[0,0,0,0,0]]))
    self.checkarray(v[1],DM([[1,2,0,0,0]]))
    self.checkarray(v[2],DM([[3,4,5,0,0]]))
    self.checkarray(v[3],DM([[6,7,8,9,0]]))
    self.checkarray(v[4],DM([[10,11,12,13,14]]))

    v = vertsplit(a,[0,2,4,5])
    self.assertEqual(len(v),3)
    self.checkarray(v[0],DM([[0,0,0,0,0],[1,2,0,0,0]]))
    self.checkarray(v[1],DM([[3,4,5,0,0],[6,7,8,9,0]]))
    self.checkarray(v[2],DM([[10,11,12,13,14]]))

    v = vertsplit(a,[0,0,3,5])
    self.assertEqual(len(v),3)
    self.assertEqual(v[0].size1(),0)
    self.assertEqual(v[0].size2(),5)
    self.checkarray(v[1],DM([[0,0,0,0,0],[1,2,0,0,0],[3,4,5,0,0]]))
    self.checkarray(v[2],DM([[6,7,8,9,0],[10,11,12,13,14]]))

  def test_horzsplit(self):
    a = DM(Sparsity.upper(5),list(range(int(5*6/2)))).T
    v = horzsplit(a,[0,2,4,5])

    self.assertEqual(len(v),3)
    self.checkarray(v[0],DM([[0,0],[1,2],[3,4],[6,7],[10,11]]))
    self.checkarray(v[1],DM([[0,0],[0,0],[5,0],[8,9],[12,13]]))
    self.checkarray(v[2],DM([[0],[0],[0],[0],[14]]))

    v = horzsplit(a)
    self.assertEqual(len(v),a.size1())
    self.checkarray(v[0],DM([0,1,3,6,10]))
    self.checkarray(v[1],DM([0,2,4,7,11]))
    self.checkarray(v[2],DM([0,0,5,8,12]))
    self.checkarray(v[3],DM([0,0,0,9,13]))
    self.checkarray(v[4],DM([0,0,0,0,14]))

    v = horzsplit(a,[0,2,4,5])
    self.assertEqual(len(v),3)
    self.checkarray(v[0],DM([[0,0],[1,2],[3,4],[6,7],[10,11]]))
    self.checkarray(v[1],DM([[0,0],[0,0],[5,0],[8,9],[12,13]]))
    self.checkarray(v[2],DM([[0],[0],[0],[0],[14]]))

    v = horzsplit(a,[0,0,3,5])
    self.assertEqual(len(v),3)
    self.assertEqual(v[0].size1(),5)
    self.assertEqual(v[0].size2(),0)
    self.checkarray(v[1],DM([[0,0,0],[1,2,0],[3,4,5],[6,7,8],[10,11,12]]))
    self.checkarray(v[2],DM([[0,0],[0,0],[0,0],[9,0],[13,14]]))

  def test_blocksplit(self):
    a = DM(Sparsity.upper(5),list(range(int(5*6/2)))).T
    v = blocksplit(a,[0,2,4,5],[0,1,3,5])

    self.checkarray(v[0][0],DM([0,1]))
    self.checkarray(v[0][1],DM([[0,0],[2,0]]))
    self.checkarray(v[1][0],DM([3,6]))
    self.checkarray(blockcat(v),a)

  def test_solve(self):
    import random

    spA = [
      Sparsity.dense(1,1)
    ]

    for n in range(2,5):
      spA+= [
        Sparsity.diag(n),
        Sparsity.dense(n,n),
        Sparsity.lower(n),
        Sparsity.lower(n).T,
        Sparsity.banded(n,1),
        diagcat(*[Sparsity.diag(n),Sparsity.dense(n,n)]),
        diagcat(*[Sparsity.diag(n),Sparsity.lower(n)]),
        diagcat(*[Sparsity.diag(n),Sparsity.lower(n).T]),
        diagcat(*[Sparsity.lower(n),Sparsity.lower(n).T]),
        Sparsity.diag(n)+Sparsity.rowcol([0],[n-1],n,n),
        Sparsity.diag(n)+Sparsity.rowcol([0,n-1],[n-1,0],n,n),
        Sparsity.diag(n)+Sparsity.triplet(n,n,[0],[n-1]),
        Sparsity.diag(n)+Sparsity.triplet(n,n,[0,n-1],[n-1,0]),
      ]

    for sA in spA:

      random.seed(1)
      a = DM(sA,[random.random() for i in range(sA.nnz())])
      A = SX.sym("a",a.sparsity())
      for sB in [ Sparsity.dense(a.size1(),1), vertcat(Sparsity.dense(1,1),Sparsity(a.size1()-1,1)),Sparsity.lower(a.size1()),Sparsity.lower(a.size1()).T]:

        b = DM(sB,[random.random() for i in range(sB.nnz())])
        B = SX.sym("B",b.sparsity())
        C = solve(A,B)

        f = Function("f", [A,B],[C])


        c = f(a,b)

        c_ref = DM(linalg.solve(a,b))
        c_ref = sparsify(c_ref)

        print(sA.dim(), sB.dim())



        #try:
        print("foo",c,c_ref)
        self.checkarray(c,c_ref)
        #self.assertTrue(min((IM.ones(c_ref.sparsity())-IM.ones(c.sparsity())).nonzeros())==0)
        #except Exception as e:
        #  c.print_dense()
        #  print("sol:")
        #  c.sparsity().spy()
        #  print("ref:")
        #  c_ref.sparsity().spy()
        #  c_ref.print_dense()
        #  a.print_dense()
        #  raise e

  def test_kron(self):
    a = sparsify(DM([[1,0,6],[2,7,0]]))
    b = sparsify(DM([[1,0,0],[2,3,7],[0,0,9],[1,12,13]]))

    c_ = c.kron(a,b)

    self.assertEqual(c_.size1(),a.size1()*b.size1())
    self.assertEqual(c_.size2(),a.size2()*b.size2())
    self.assertEqual(c_.nnz(),a.nnz()*b.nnz())

    self.checkarray(c_,numpy.kron(a,b))

  def test_vec_kron(self):
    A = SX.sym("A",2,3)
    B = SX.sym("B",4,5)
    P = SX.sym("P",A.size2(),B.size1())

    f = Function("f", [vec(P.T),A,B],[vec(mtimes([A,P,B]).T)])

    J = jacobian_old(f, 0, 0)
    J_in = []
    J_in.append(numpy.random.rand(*vec(P.T).shape))
    J_in.append(numpy.random.rand(*A.shape))
    J_in.append(numpy.random.rand(*B.shape))

    res, _  =  J(*J_in)

    ref =  kron(J_in[1],J_in[2].T)

    self.checkarray(res,ref)

  def test_repmat(self):
    a = DM([[1,2],[3,4],[5,6]])
    self.checkarray(repmat(a,2,3),kron(DM.ones(2,3),a))

  def test_triu(self):
    a = DM([[1,2],[3,4]])
    b = triu(a)
    self.checkarray(b, DM([[1,2],[0,4]]) )


  def test_tril(self):
    a = DM([[1,2],[3,4]])
    b = tril(a)
    self.checkarray(b, DM([[1,0],[3,4]]) )

  def test_nz(self):
    a = sparsify(DM([[1,2],[0,0],[3,4]]))
    self.checkarray(a.nz[:], DM([1,3,2,4]) )
    self.checkarray(len(a.nz), 4 )
    self.checkarray(a.nz[:-1], DM([1,3,2]) )
    self.checkarray(a.nz[0], DM([1]) )

  def test_norm_inf_mul(self):
    numpy.random.seed(0)

    A = numpy.random.random((10,2))
    B = numpy.random.random((2,8))

    self.checkarray(norm_inf_mul(A,B),norm_inf(mtimes(A,B)))
    self.checkarray(DM(norm_0_mul(A,B)),mtimes(A,B).nnz())

    # Sparse
    for i in range(5):
      A[numpy.random.randint(A.shape[0]),numpy.random.randint(A.shape[1])] = 0
      B[numpy.random.randint(B.shape[0]),numpy.random.randint(B.shape[1])] = 0

    A = sparsify(A)
    B = sparsify(B)

    self.checkarray(norm_inf_mul(A,B),norm_inf(mtimes(A,B)))
    self.checkarray(DM(norm_0_mul(A,B)),mtimes(A,B).nnz())


    A = numpy.random.random((8,2))
    B = numpy.random.random((2,10))

    self.checkarray(norm_inf_mul(A,B),norm_inf(mtimes(A,B)))
    self.checkarray(DM(norm_0_mul(A,B)),mtimes(A,B).nnz())

    # Sparse
    for i in range(5):
      A[numpy.random.randint(A.shape[0]),numpy.random.randint(A.shape[1])] = 0
      B[numpy.random.randint(B.shape[0]),numpy.random.randint(B.shape[1])] = 0

    A = sparsify(A)
    B = sparsify(B)

    self.checkarray(norm_inf_mul(A,B),norm_inf(mtimes(A,B)))
    self.checkarray(DM(norm_0_mul(A,B)),mtimes(A,B).nnz())

  def  test_mul3_issue_1465(self):
    with self.assertRaises(Exception):
      w = SX.sym("w",2,1)
      Q = np.eye(2)
      mtimes(w.T,Q,w)

  def test_chol(self):
    numpy.random.seed(0)

    for i in range(4):
      A = numpy.random.random((3,3))
      H = mtimes(A,A.T)

      R = chol(H)

      assert R.is_triu()
      self.checkarray(mtimes(R.T,R),H)
  def test_skew(self):
    x = DM([1,7,13])
    self.checkarray(inv_skew(skew(x)),x)
    y = DM([0.2,0.9,0.4])
    self.checkarray(mtimes(skew(x),y),cross(x,y))

  def test_nz_overflow(self):
    d = DM([2,3])
    r = d.nz[:]
    self.checkarray(r,d)

  def test_DMcrash(self):
    with self.assertRaises(Exception):
      DM([DM([1,2]),DM([1,2])])
    a = DM([DM([1]),DM([2])])
    self.checkarray(a,DM([1,2]))

  def test_sparsity_operation(self):
    L = [DM(1), DM(Sparsity(1,1),1), DM(Sparsity(2,1),1), DM(Sparsity.dense(2,1),1)]

    for a in L:
      for b in L:
        c = a*b

        if a.nnz()==0 or b.nnz()==0:
          self.assertTrue(c.nnz()==0)
        else:
          self.assertTrue(c.nnz()>0)

    self.assertTrue(sum2(DM(Sparsity(1,1),1)).nnz()==0)

  def test_matlab_operations(self):

    data = [ np.array([[1,3],[11,17]]) , np.array([[1,3]]) ,np.array([[1],[3]]), np.array([[3]])]

    for A in data:
      B = reshape(DM(A),A.shape)
      #self.checkarray(np.cumsum(A),cumsum(B))
      self.checkarray(np.cumsum(A,0),cumsum(B,0))
      self.checkarray(np.cumsum(A,1),cumsum(B,1))

      Bs = MX.sym("B",B.shape)
      A0 = cumsum(B,0)
      A1 = cumsum(B,1)

      self.checkarray(np.cumsum(A,0),evalf(cumsum(MX(B),0)))
      self.checkarray(np.cumsum(A,1),evalf(cumsum(B,1)))

      f = Function("f",[Bs],[A0,A1])
      self.checkfunction(f,f.expand(),inputs = [B])
      self.check_codegen(f,inputs=[B],check_serialize=True)

      #self.checkarray(np.diff(A),diff(B))

      #self.checkarray(np.diff(A,1),diff(B,1))
      #self.checkarray(np.diff(A,2),diff(B,2))

      self.checkarray(np.diff(A,1,0),diff(B,1,0))
      #self.checkarray(np.diff(A,1,1),diff(B,1,1))


  def test_singular_repmat(self):
    for X in [DM, SX, MX, Sparsity]:
      for n_b in [0,2]:
        for m_b in [0,2]:
          b = X(n_b, m_b)

          for n in [0,3]:
            for m in [0,3]:
              self.assertEqual(repmat(b, n, m).shape,(n_b * n, m_b * m))


  def test_serialize(self):
    for a in [DM(), DM(2), DM([1,2]),DM([[1,2],[3,4],[5,6]])]:
      b = DM.deserialize(a.serialize())
      self.checkarray(a,b)

  def test_iterable(self):
    a = DM([1,2,3])
    b = list(iter(a.nz))
    self.checkarray(a,DM(b))

    with self.assertInException("CasADi matrices are not iterable"):
      iter(a)

  def test_from_file(self):
    a = sparsify(DM([[1,0,-6,5,0],[4,0,-4e-301,9.3e-18,0]]))

    a.print_dense()

    a[0,4] = 0
    a[1,1] = 0
    a.to_file("test.mtx")

    with self.assertInException("foobar.mtx"):
      DM.from_file("foobar.mtx")
    b = DM.from_file("test.mtx")

    self.assertTrue(a.sparsity()==b.sparsity())
    self.checkarray(a,b)

    a.to_file("test.txt")
    b = DM.from_file("test.txt")
    self.checkarray(a,b)

    for s in ["1 00 -6 5 0\n4 0 -4e-301 9.3e-18 00\n",
              "1 00 -6 5 0\n4 0 -4e-301 9.3e-18 00",
              "1 00 -6 5 0 \n4 0 -4e-301 9.3e-18 00   ",
              "1\t00\t-6\t5\t0\t\n4 0 -4e-301 9.3e-18 00\n",
              "1 00 -6 5 0\n%foobar\n4 0 -4e-301 9.3e-18 00\n"]:

      with open("test2.txt","w") as f:
        f.write(s)
      b = DM.from_file("test2.txt")
      self.assertTrue(a.sparsity()==b.sparsity())
      self.assertTrue(float(norm_1(a-b))==0)

    with open("test.txt","w") as f:
      f.write("1 aa 6\n4 8 9e-18\n")


    with self.assertInException("Parsing"):
      DM.from_file("test.txt")

    a = DM([3, inf, -inf, np.nan])
    a.to_file("test.txt")
    b = DM.from_file("test.txt")
    self.checkarray(a,b)
  def test_norm_2(self):
    a = np.array([1,2,3])
    r = np.linalg.norm(a)
    for M in [DM,MX,SX]:
      self.checkarray(evalf(norm_2(M(a))),r)
      self.checkarray(evalf(norm_2(M(a).T)),r)

  def test_ldl(self):
    numpy.random.seed(1)
    H = diagcat(DM.rand(5,5),DM.rand(5,5))
    H = H+H.T+2*DM.eye(10)

    p = np.random.permutation(10)
    H = H[p,p]


    [D,Lt,p] = ldl(H)
    P = DM.eye(10)[:,p]

    F = mtimes(mtimes(sqrt(diag(D)),DM.eye(10)+Lt),P.T)
    print(H-mtimes(F.T,F))
    self.assertTrue(norm_fro(H-mtimes(F.T,F))<=1e-14)


  def test_im_bugs(self):
    a = vertcat(1,2)
    self.assertTrue(isinstance(a,DM))
    self.checkarray(c.linspace(1,3,10),c.linspace(1.0,3.0,10))
  
  def test_permutation(self):
    n = 10
    numpy.random.seed(1)
    p = np.random.permutation(n)
    S = Sparsity.permutation(p)
    self.checkarray(DM(p).T,S.permutation_vector())
    self.assertTrue(S.is_permutation())
    v = DM.rand(n)
    self.checkarray(mtimes(S,v), v[p])
    S = Sparsity.permutation(p, True)
    self.checkarray((mtimes(S,v))[p], v)

  def test_sparsity_orthonormality(self):
    alltests = [('is_orthonormal_rows',True),('is_orthonormal_columns',True),('is_selection',True),('is_orthonormal_rows',False),('is_orthonormal_columns',False),('is_selection',False),('is_permutation',),('is_orthonormal',False),('is_orthonormal',True)]
    
    def check_properties(S,prop):
      for p in prop:
        assert p in alltests
      for t in alltests:
        quality = t in prop
        method = getattr(S,t[0])
        result = method(*t[1:])
        self.assertEqual(quality,result,"Call %s expected to return %s" % (str(t),str(quality)))
        
    for S in [sparsify(DM([[0,0,0,0,0],
                           [0,0,0,0,0],
                           [0,0,0,0,0],
                           [0,0,0,0,0]])),
              sparsify(DM([[0,0,0,0,0],
                           [0,1,0,0,0],
                           [0,0,0,0,0],
                           [0,0,0,0,0]])),
              sparsify(DM([[0,0,0,0,0],
                           [0,1,0,0,0],
                           [0,0,0,1,0],
                           [0,0,0,0,0]]))]:
      check_properties(S.sparsity(), [('is_orthonormal_rows',True),('is_orthonormal_columns',True),('is_selection',True),('is_orthonormal',True)]) 

    for S in [sparsify(DM([[0,0,0,0,0],
                           [0,1,1,0,0],
                           [0,0,0,0,0],
                           [0,0,0,0,0]])),
              sparsify(DM([[0,1,0,0],
                           [1,0,0,0],
                           [0,1,1,0],
                           [0,0,0,0],
                           [0,0,0,1]])),
              sparsify(DM([[0,0,0,0,0],
                           [0,1,0,0,0],
                           [0,1,0,0,0],
                           [0,0,0,0,0]]))]:
      check_properties(S.sparsity(), []) 
    
    S = sparsify(DM([[0,1,0,0,0],
                     [1,0,0,0,0],
                     [0,0,1,0,0],
                     [0,0,0,1,0]]))
    check_properties(S.sparsity(), [('is_orthonormal_rows',True),('is_orthonormal_columns',True),('is_selection',True),('is_orthonormal_rows',False),('is_selection',False),('is_orthonormal',True)])
    
    S = sparsify(DM([[0,1,0,0],
                     [1,0,0,0],
                     [0,0,1,0],
                     [0,0,0,1]]))
    check_properties(S.sparsity(), [('is_orthonormal_rows',True),('is_orthonormal_columns',True),('is_selection',True),('is_orthonormal_rows',False),('is_orthonormal_columns',False),('is_selection',False),('is_permutation',),('is_orthonormal',True),('is_orthonormal',False)])

    S = sparsify(DM([[0,1,0,0],
                     [1,0,0,0],
                     [0,0,1,0],
                     [0,0,0,0],
                     [0,0,0,1]]))
    check_properties(S.sparsity(), [('is_orthonormal_rows',True),('is_orthonormal_columns',True),('is_selection',True),('is_orthonormal_columns',False),('is_orthonormal',True)])
    
  def test_permutation_solve(self):
    n = 10
    numpy.random.seed(1)
    DM.rng(1)
    p = np.random.permutation(n)
    P = Sparsity.permutation(p)
    fs = []
    for X in [SX,MX]:
      x = X.sym("x",P)
      b = X.sym("b",n)
      assert x.sparsity().is_orthonormal()
      nz = sparsity_cast(x, Sparsity.dense(n))
      f = Function("f",[x,b],[solve(x,b)])
      f.generate('f.c')
      fs.append(f)    
    x0 = DM.rand(x.sparsity())
    b0 = DM.rand(b.sparsity())
    res0 = fs[0](x0,b0)
    res1 = fs[1](x0,b0)
    self.checkarray(res0,res1)

  def test_permutation_inv(self):
    n = 10
    numpy.random.seed(1)
    DM.rng(1)
    p = np.random.permutation(n)
    P = Sparsity.permutation(p)
    fs = []
    for X in [SX,MX]:
      x = X.sym("x",P)
      f = Function("f",[x],[solve(x,X.eye(n))])
      f.generate('f.c')
      fs.append(f)    
    x0 = DM(P,DM.rand(n))
    res0 = fs[0](x0)
    res1 = fs[1](x0)
    self.checkarray(res0,res1)

    fs = []
    for X in [SX,MX]:
      x = X.sym("x",P)
      f = Function("f",[x],[inv(x)])
      f.disp(True)
      fs.append(f)    
    x0 = DM(P,DM.rand(n))
    res0 = fs[0](x0)
    res1 = fs[1](x0)
    self.checkarray(res0,res1)

    
  def test_mac_simplify(self):
    n = 10
    numpy.random.seed(1)
    DM.rng(1)
    p = np.random.permutation(n)
    P = Sparsity.permutation(p)
    fs = []
    for X in [SX,MX]:
      b = X.sym("b",n)
      x = X.sym("x",P)
      f = Function("f",[x,b],[mtimes(x,b)])
      f.generate('f.c')
      fs.append(f)  
    x0 = DM(P,DM.rand(n))
    b0 = DM.rand(b.sparsity())
    res0 = fs[0](x0,b0)
    res1 = fs[1](x0,b0)
    self.checkarray(res0,res1)
    

  def test_inplace(self):
    n = 10
    x = MX.sym("x",n)
    y = MX(Sparsity.diag(n),x)
    f = Function('f',[x],[y])
    self.assertTrue(f.sz_w()==n)


  def test_horzsplit_n(self):
    self.assertTrue(len(horzsplit_n(DM.rand(1,6),2))==2)
    self.assertTrue(len(vertsplit_n(DM.rand(6),2))==2)
    self.assertTrue(len(horzsplit_n(DM(0,0),2))==2)
    self.assertTrue(len(vertsplit_n(DM(0,0),2))==2)
    with self.assertRaises(Exception):
       horzsplit_n(DM.rand(1,5),2)
    with self.assertRaises(Exception):
       vertsplit_n(DM.rand(5),2)
       
if __name__ == '__main__':
    unittest.main()



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs047():
    # The optimal objective is (if given in):
    x_opt = cs.DM([1,1,1,1,1])
    f_opt = 0
    x = cs.MX.sym('x', 5)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(5, 1)
    lbx = -cs.inf*cs.DM.ones(5, 1)
    ubx = cs.inf*cs.DM.ones(5, 1)
    g = cs.MX.zeros(3, 1)
    lbg = -cs.inf*cs.DM.ones(3, 1)
    ubg = cs.inf*cs.DM.ones(3, 1)
    
    x0[0]= 2
    x0[1]= cs.sqrt(2)
    x0[2]= -1
    x0[3]= 2-cs.sqrt(2)
    x0[4]= 1/2
    
    lbg[0] =  3
    ubg[0] =  3
    g[0] = x[0] + x[1]**2 + x[2]**3
    lbg[1] =  1
    ubg[1] =  1
    g[1] = x[1] - x[2]**2 + x[3]
    lbg[2] =  1
    ubg[2] =  1
    g[2] = x[0]*x[4]
    obj = (x[0]-x[1])**2 + (x[1]-x[2])**3 + (x[2]-x[3])**4 + (x[3]-x[4])**4
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs044():
    # The optimal objective is (if given in):
    x_opt = cs.DM([0, 3, 0, 4])
    f_opt = -15
    x = cs.MX.sym('x', 4)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(4, 1)
    lbx = -cs.inf*cs.DM.ones(4, 1)
    ubx = cs.inf*cs.DM.ones(4, 1)
    g = cs.MX.zeros(6, 1)
    lbg = -cs.inf*cs.DM.ones(6, 1)
    ubg = cs.inf*cs.DM.ones(6, 1)
    
    x0[0:4]= 0
    lbx[0:4]=0
    
    ubg[0] = 8
    g[0] = x[0] + 2*x[1]
    ubg[1] = 12
    g[1] = 4*x[0] + x[1]
    ubg[2] = 12
    g[2] = 3*x[0] + 4*x[1]
    ubg[3] = 8
    g[3] = 2*x[2] + x[3]
    ubg[4] = 8
    g[4] = x[2] + 2*x[3]
    ubg[5] = 5
    g[5] = x[2] + x[3]
    obj = x[0] - x[1] - x[2] - x[0]*x[2] + x[0]*x[3] + x[1]*x[2] - x[1]*x[3]
    
    
    f = obj
    print('hs044: Other local minimum is f_opt=-13 at (3, 0, 4, 0).')
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import unittest
from types import *
from helpers import *

solvers= []
try:
  load_rootfinder("kinsol")
  solvers.append(("kinsol",{"abstol":1e-10},[]))
except:
  pass
try:
  load_nlpsol("ipopt")
  solvers.append(("nlpsol",{"nlpsol": "ipopt","nlpsol_options":{"print_time": False,"ipopt": {"print_level": 0}}},[]))
except:
  pass
try:
  solvers.append(("newton",{},[]))
except:
  pass

solvers.append(("fast_newton",{},("codegen")))

print(solvers)

class ImplicitFunctiontests(casadiTestCase):

  @memory_heavy()
  def test_linear(self):
    for Solver, options, features in solvers:
      self.message(Solver)
      x=SX.sym("x",2)
      A_ = DM([[1,2],[3,2.1]])
      C_ = DM([[1.6,2.1],[1,1.3]])
      b_ = DM([0.7,0.6])
      f=Function("f", [x],[mtimes(A_,x)-b_, mtimes(C_,x)])
      solver=rootfinder("solver", Solver, f, options)
      solver_out = solver(0)

      refsol = Function("refsol", [x],[solve(A_,b_), mtimes(C_,solve(A_,b_))])
      self.checkfunction(solver,refsol,inputs=[0],digits=10)
      if "newton" in Solver: self.check_serialize(solver,inputs=[0])
      if "codegen" in features: self.check_codegen(solver,inputs=[0])

      A = SX.sym("A",2,2)
      b = SX.sym("b",2)
      f=Function("f", [x,A,b],[mtimes(A,x)-b])
      solver=rootfinder("solver", Solver, f, options)
      solver_in = [0]*3
      solver_in[1]=A_
      solver_in[2]=b_

      refsol = Function("refsol", [x,A,b],[solve(A,b)])
      self.checkfunction(solver,refsol,inputs=solver_in,digits=10)
      if "newton" in Solver: self.check_serialize(solver,inputs=solver_in)
      if "codegen" in features: self.check_codegen(solver,inputs=solver_in)

      A = SX.sym("A",2,2)
      b = SX.sym("b",2)
      f=Function("f", [x,A,b],[mtimes(A,x)-b,mtimes(C_,x)])
      for ad_weight_sp in [0,1]:
        for ad_weight in [0,1]:
          print(ad_weight, ad_weight_sp)
          options2 = dict(options)
          options2["ad_weight_sp"] = ad_weight_sp
          options2["ad_weight"] = ad_weight
          solver=rootfinder("solver", Solver, f, options2)
          solver_in = [0]*3
          solver_in[1]=A_
          solver_in[2]=b_

          refsol = Function("refsol", [x,A,b],[solve(A,b),mtimes(C_,solve(A,b))])
          self.checkfunction(solver,refsol,inputs=solver_in,digits=10)
          if "newton" in Solver: self.check_serialize(solver,inputs=solver_in)
      if "codegen" in features: self.check_codegen(solver,inputs=solver_in)

  def test_missing_symbols(self):
    for Solver, options, features in solvers:
      self.message(Solver)
      x=SX.sym("x")
      p=SX.sym("p")
      f=Function("f", [x], [sin(x+p)],{"allow_free":True})
      with self.assertInException("[p] are free"):
        rootfinder("solver", Solver, f, options)

  def test_scalar1(self):
    self.message("Scalar implicit problem, n=0")
    for Solver, options, features in solvers:
      self.message(Solver)
      x=SX.sym("x")
      f=Function("f", [x], [sin(x)])
      solver=rootfinder("solver", Solver, f, options)
      solver_out = solver(0)

      refsol = Function("refsol", [x], [ceil(x/pi-0.5)*pi])
      self.checkfunction(solver,refsol,inputs=[6],digits=10)

  def test_unsolved_stats(self):
    for Solver, options, features in solvers:
      self.message(Solver)
      x=SX.sym("x")
      f=Function("f", [x], [sin(x)])
      solver=rootfinder("solver", Solver, f, options)
      try:
          solver.stats()
      except:
          pass # no segfault
     
  def test_scalar2(self):
    self.message("Scalar implicit problem, n=1")
    for Solver, options, features in solvers:
      self.message(Solver)
      message = Solver
      x=SX.sym("x")
      y=SX.sym("y")
      n=0.2
      f=Function("f", [y,x], [x-arcsin(y)])
      solver=rootfinder("solver", Solver, f, options)
      refsol = Function("refsol", [y,x], [sin(x)])
      self.checkfunction(solver,refsol,inputs=[0,n],digits=6,sens_der=False,failmessage=message)

  def test_scalar2_indirect(self):
    for Solver, options, features in solvers:
      self.message(Solver)
      message = Solver
      x=SX.sym("x")
      y=SX.sym("y")
      n=0.2
      f=Function("f", [y,x], [x-arcsin(y)])
      solver=rootfinder("solver", Solver, f, options)

      X = MX.sym("X")
      R = solver(MX(),X)

      trial = Function("trial", [X], [R])
      refsol = Function("refsol", [x],[sin(x)])
      self.checkfunction(trial,refsol,inputs=[n],digits=6,sens_der=False,failmessage=message)

  def test_large(self):
    for Solver, options, features in solvers:
      if 'kinsol' in str(Solver): continue
      if 'newton' in str(Solver): continue

      message = Solver
      N = 5
      s = Sparsity.lower(N)
      x=SX.sym("x",s)

      y=SX.sym("y",s)
      y0 = DM(Sparsity.diag(N),0.1)

      f=Function("f", [y.nz[:],x.nz[:]],[((mtimes((x+y0),(x+y0).T)-mtimes((y+y0),(y+y0).T))[s]).nz[:]])
      options2 = dict(options)
      options2["constraints"] = [1]*s.nnz()
      solver=rootfinder("options2", Solver, f, options2)

      X = MX.sym("X",x.sparsity())
      R = solver(MX(),X.nz[:])

      trial = Function("trial", [X],[R])
      trial_in = DM(trial.sparsity_in(0),[abs(cos(i)) for i in range(x.nnz())])
      trial_out = trial(trial_in)

      f_in = [trial_out, trial_in.nz[:]]
      f_out = f(*f_in)

      f_in = [trial_in.nz[:], trial_in.nz[:]]
      f_out = f(*f_in)

      refsol = Function("refsol", [X],[X.nz[:]])
      refsol_in = [0]*refsol.n_in();refsol_in[0]=trial_in[0]

      self.checkfunction(trial,refsol,inputs=refsol_in,digits=6,sens_der=False,evals=1,failmessage=message)

  @known_bug()
  def test_vector2(self):
    self.message("Scalar implicit problem, n=1")
    for Solver, options, features in solvers:
      self.message(Solver)
      message = Solver
      x=SX.sym("x")
      y=SX.sym("y",2)
      y0 = DM([0.1,0.4])
      yy = y + y0
      n=0.2
      f=Function("f", [y,x],[vertcat(x-arcsin(yy[0]),yy[1]**2-yy[0])])
      solver=rootfinder("solver", Solver, f, options)

      refsol = Function("refsol", [y,x],[vertcat(sin(x),sqrt(sin(x)))-y0]) # ,sin(x)**2])
      self.checkfunction(solver,refsol,inputs=[n,0],digits=4,sens_der=False,failmessage=message)

  def testKINSol1c(self):
    self.message("Scalar KINSol problem, n=0, constraint")
    x=SX.sym("x")
    f=Function("f", [x],[sin(x)])
    solver=rootfinder("solver", "kinsol", f, {"constraints":[-1]})
    solver_out = solver(-6)
    self.assertAlmostEqual(solver_out[0],-2*pi,5)

  def test_constraints(self):
    for Solver, options, features in solvers:
      if 'kinsol' in str(Solver): continue
      if 'newton' in str(Solver): continue

      print(Solver, options)
      x=SX.sym("x",2)
      f=Function("f", [x],[vertcat(*[mtimes((x+3).T,(x-2)),mtimes((x-4).T,(x+vertcat(*[1,2])))])])
      options2 = dict(options)
      options2["constraints"] = [-1,0]
      solver=rootfinder("solver", Solver, f, options2)
      solver_out = solver(0)

      self.checkarray(solver_out,DM([-3.0/50*(sqrt(1201)-1),2.0/25*(sqrt(1201)-1)]),digits=6)

      f=Function("f", [x],[vertcat(*[mtimes((x+3).T,(x-2)),mtimes((x-4).T,(x+vertcat(*[1,2])))])])
      options2 = dict(options)
      options2["constraints"] = [1,0]
      solver=rootfinder("solver", Solver, f, options2)
      solver_out = solver(0)

      self.checkarray(solver_out,DM([3.0/50*(sqrt(1201)+1),-2.0/25*(sqrt(1201)+1)]),digits=6)

  def test_implicitbug(self):
    # Total number of variables for one finite element
    X0 = MX.sym("X0")
    V = MX.sym("V")

    V_eq = vertcat(*[V[0]-X0])

    # Root-finding function, implicitly defines V as a function of X0 and P
    vfcn = Function("vfcn", [V,X0], [V_eq], {"ad_weight":0, "ad_weight_sp":1})

    # Convert to SX to decrease overhead
    vfcn_sx = vfcn.expand('vfcn_sx', {"ad_weight":0, "ad_weight_sp":1})

    # Create a implicit function instance to solve the system of equations
    ifcn = rootfinder("ifcn", "newton", vfcn_sx, {"linear_solver":"csparse"})

    #ifcn = Function('I', [X0],[vertcat(*[X0])])
    [V] = ifcn.call([0,X0],True)

    f = 1  # fails

    F = Function("F", [X0], [f*X0+V], {"ad_weight":0, "ad_weight_sp":1})

    # Test values
    x0_val  = 1

    J = jacobian_old(F, 0, 0)
    J_out = J(x0_val)
    print(J_out[0])
    print(J)

    self.checkarray(J_out[0],DM([2]))

  def test_extra_outputs(self):
    x = SX.sym("x")
    a = SX.sym("a")
    f = Function("f", [x,a],[tan(x)-a,sqrt(a)*x**2 ])
    for Solver, options, features in solvers:
      print(Solver)
      options2 = dict(options)
      options2["ad_weight_sp"] = 1
      solver=rootfinder("solver", Solver, f, options2)
      solver_in = [0.1,0.3]

      refsol = Function("refsol", [x,a],[arctan(a),sqrt(a)*arctan(a)**2])
      self.checkfunction(solver,refsol,inputs=solver_in,digits=5)
      if "codegen" in features: self.check_codegen(solver,inputs=solver_in)

    x = SX.sym("x",2)
    a = SX.sym("a",2)
    f = Function("f", [x,a],[tan(x)-a,sqrt(a)*x**2 ])

  def test_no_success(self):

    x=SX.sym("x")
    y=SX.sym("y")

    for Solver, options, features in solvers:
      opts = dict(options)
      opts["error_on_fail"] = False
      solver = rootfinder("solver",Solver,{'x':vertcat(x,y), 'g':vertcat(sin(x)-2,sin(y)-2)},opts)
      solver(x0=0)
      self.assertFalse(solver.stats()["success"])

      opts = dict(options)
      solver = rootfinder("solver",Solver,{'x':vertcat(x,y), 'g':vertcat(sin(x)-2,sin(y)-2)},opts)
      with self.assertInException("process"):
        solver(x0=0)

  def test_loop(self):
    x=SX.sym("x")
    for Solver, options, features in solvers:
      solver = rootfinder("solver",Solver,{"x":x,"g":x**3-2*x+2},options)
      if Solver=="kinsol": continue
      if Solver=="nlpsol": continue
      if Solver=="fast_newton": continue
      print(Solver)
      print(solver)
      res = solver(x0=0)["x"]
      self.checkarray(res,-1.7692923542386)

  def test_segfault_codegen(self):
    # Symbols
    x = MX.sym("x")
    y = MX.sym("y")

    # B-Spline interpolant
    u = np.linspace(1, 5, 10)
    v = u ** 2
    f = interpolant("interp", "bspline", [u], v, {})

    # Extrapolated interpolant, suitable for rootfinding later on
    f = Function(
        "f",
        [x],
        [if_else(x < u[0], x ** 2, if_else(x > u[-1], x ** 2, f(x), False), False)],
    )

    # Use rootfinder to find x such that x**2=y
    res = Function("res", [x, y], [y - f(x)])
    rf = rootfinder("rf", "fast_newton", res)

    g = Function("g",[x],[x - rf(x, x)])
    self.check_codegen(g,[1.01])
    
    
  def test_generic(self):
    a = SX.sym("a")
    b = SX.sym("b")
    c = SX.sym("c")
    
    f = Function("f",[a,b,c],[a**2-b,b**2-c,a*b*c+1],["a","b","c"],["f","g","h"])

    a0 = 1
    b0 = 2
    c0 = 3
     
    rf = rootfinder("rf","newton",f,{"implicit_input":0,"implicit_output":0})
     
    res = rf(a0=a0,b=b0,c=c0)
    self.assertAlmostEqual(res["a"]**2-b0,0,10)
    self.assertAlmostEqual(b0**2-c0-res["g"],0,10)
    self.assertAlmostEqual(res["a"]*b0*c0+1-res["h"],0,10)
    print(rf)
    jac_g_x = rf.get_function("jac_g_x")
    self.assertTrue(jac_g_x.name_in()==["a","b","c"])
    self.assertTrue(jac_g_x.name_out()==["jac_f_a","f","g","h"])
    rf = rootfinder("rf","newton",f,{"implicit_input":1,"implicit_output":0})
    res = rf(a=a0,b0=b0,c=c0)
    self.assertAlmostEqual(a0**2-res["b"],0,10)
    self.assertAlmostEqual(res["b"]**2-c0-res["g"],0,10)
    self.assertAlmostEqual(a0*res["b"]*c0+1-res["h"],0,10)

    jac_g_x = rf.get_function("jac_g_x")
    self.assertTrue(jac_g_x.name_in()==["a","b","c"])
    self.assertTrue(jac_g_x.name_out()==["jac_f_b","f","g","h"])
    
    rf = rootfinder("rf","newton",f,{"implicit_input":2,"implicit_output":1})
    res = rf(a=a0,b=b0,c0=c0)
    self.assertAlmostEqual(a0**2-b0-res["f"],0,10)
    self.assertAlmostEqual(b0**2-res["c"],0,10)
    self.assertAlmostEqual(a0*b0*res["c"]+1-res["h"],0,10)
    jac_g_x = rf.get_function("jac_g_x")
    self.assertTrue(jac_g_x.name_in()==["a","b","c"])
    self.assertTrue(jac_g_x.name_out()==["jac_g_c","f","g","h"])

    rf = rootfinder("rf","newton",{"x":a,"p":vertcat(b,c),"g":a**2-b})
    res = rf(x0=a0,p=vertcat(b0,c0))
    self.assertAlmostEqual(res["x"]**2-b0,0,10)
    jac_g_x = rf.get_function("jac_g_x")
    self.assertTrue(jac_g_x.name_in()==["x","p"])
    self.assertTrue(jac_g_x.name_out()==["jac_g_x","g"])
    
if __name__ == '__main__':
    unittest.main()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#

from casadi import *

# CasADi provides a mechanism to add assertions in an MX expression graph
# This can be useful to debug yor code, e.g. debugging why the end-result of a computation yields NaN

# Consider this example:

x = MX.sym("x")
y = sin(x)
z = sqrt(y)

f = Function("f", [x], [z])

z0 = f(5)

print(z0)

# For some mysterious reason we get NaN here

# Next, we add an assertion:

y = y.attachAssert(y>0, "bummer") # Add assertion here
z = sqrt(y)

f = Function("f", [x],[z])

try:
  z0 = f(5)
except Exception as e:
  print("An exception was raised here:")
  print(e)


# You can combine this with Callback to do powerful assertions

class Dummy(Callback):
  def __init__(self, name, opts={}):
    Callback.__init__(self)
    self.construct(name, opts)
  def get_n_in(self): return 1
  def get_n_out(self): return 1
  def eval(self, arg):
    import numpy
    x = arg[0]
    m = max(numpy.real(numpy.linalg.eig(blockcat([[x,-1],[-1,2]]))[0]))
    print("m=",m)
    return [int(m>2)]

foo = Dummy("foo")

y = sin(x)

y = y.attachAssert(foo(y), "you are in trouble") # Add assertion here
z = sqrt(y)

f = Function("f", [x],[z])

z0 = f(5)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import unittest
from types import *
from helpers import *
import casadi as ca

import os
            
class OCPtests(casadiTestCase):

  def fatrop_case(self,N=2, nx0=2, nu0=2, nx1=2, nu1=2, nx2=2, nu2=2, ng1=2, ng2=2, ng3=2, sp=None,eq=None):
        print("fatrop_case",N,nx0,nu0,nx1,nu1,nx2,nu2,ng1,ng2,ng3,sp,eq)
        if sp is None:
            sp = {}
        if eq is None:
            eq = set()
        nx = [nx0 ,nx1, nx2]
        nu = [nu0, nu1, nu2]
        ng = [ng1, ng2, ng3]
        
        print("nx",nx)
        print("nu",nu)
        print("ng",ng)
        
        DM.rng(1)
        
        A0 = DM.rand(nx1, nx0)
        if "A0" in sp: A0 = project(A0, sp["A0"])
        B0 = DM.rand(nx1, nu0)
        if "B0" in sp: B0 = project(B0, sp["B0"])
        C0 = DM.rand(ng1, nx0)
        if "C0" in sp: C0 = project(C0, sp["C0"])
        D0 = DM.rand(ng1, nu0)
        I0 = DM.eye(nx1)
        
        A1 = DM.rand(nx2, nx1)
        if "A1" in sp: A1 = project(A1, sp["A1"])
        B1 = DM.rand(nx2, nu1)
        if "B1" in sp: B1 = project(B1, sp["B1"])
        C1 = DM.rand(ng2, nx1)
        if "C1" in sp: C1 = project(C1, sp["C1"])
        D1 = DM.rand(ng2, nu1)
        if "D1" in sp: D1 = project(D1, sp["D1"])
        I1 = DM.eye(nx2)
       
        C2 = DM.rand(ng3, nx2)
        if "C2" in sp: C2 = project(C2, sp["C2"])
        D2 = DM.rand(ng3, nu2)
        if "D2" in sp: D2 = project(D2, sp["D2"])
        
        A = blockcat([[A0,B0,I0,DM(nx1,nu1+nx2+nu2)],[C0,D0,DM(ng1,nx1+nu1+nx2+nu2)],[DM(nx2,nx0+nu0),A1,B1,I1,DM(nx2,nu2)],[DM(ng2,nx0+nu0),C1,D1,DM(ng2,nx2+nu2)],[DM(ng3,nx0+nu0+nx1+nu1),C2,D2]])
        
        
        
        equality = [True]*nx1+["ng1" in eq]*ng1+[True]*nx2+["ng2" in eq]*ng2+["ng3" in eq]*ng3
        
        A.sparsity().spy()
        print(A)
       
        x0 = MX.sym("x0",nx0)
        u0 = MX.sym("u0",nu0)
        x1 = MX.sym("x1",nx1)
        u1 = MX.sym("u1",nu1)
        x2 = MX.sym("x2",nx2)
        u2 = MX.sym("u2",nu2)      
        
        x = vertcat(x0,u0,x1,u1,x2,u2)
        nlp = {}
        nlp["x"] = x
        nlp["g"] = DM.zeros(A.shape[0],1) + mtimes(A,x)
        
        nlp["f"] = sumsqr(x-DM.rand(x.numel(),1))
        
        a = 10
        lbg = vertcat(DM.zeros(nx1,1),-a*DM.ones(ng1,1),DM.zeros(nx2,1),-a*DM.ones(ng2,1),-a*DM.ones(ng3,1))
        ubg = vertcat(DM.zeros(nx1,1),a*DM.ones(ng1,1),DM.zeros(nx2,1),a*DM.ones(ng2,1),a*DM.ones(ng3,1))
        
        print(lbg)

        
        options = {"structure_detection": "manual", "N":N, "nx": nx, "nu":nu, "ng": ng, "equality": equality,"fatrop":{"tol":1e-7}}
        solver = nlpsol("solver","fatrop",nlp,options)
        sol = solver(lbg=lbg,ubg=ubg)

        solver = nlpsol("solver","fatrop",nlp,{"structure_detection": "none", "error_on_fail":True, "equality": equality,"fatrop":{"tol":1e-7}})
        ref = solver(lbg=lbg,ubg=ubg)
        
        for k in sol.keys():
            self.checkarray(sol[k],ref[k],failmessage=k+str(options),digits=6)

        options = {"structure_detection": "auto", "debug":True, "equality": equality,"fatrop":{"tol":1e-7}}
        print(options)
        solver = nlpsol("solver","fatrop",nlp,options)
        sol = solver(lbg=lbg,ubg=ubg)
        
        stats = solver.stats()
        print(stats)
        if nx2>0:
            self.assertTrue(stats["N"]>1)
        
        
        for k in sol.keys():
            self.checkarray(sol[k],ref[k],failmessage=k+str(options),digits=6)

  @requires_nlpsol("ipopt")
  def testdiscrete(self):
    self.message("Linear-quadratic problem, discrete, using IPOPT")
    # inspired by www.cs.umsl.edu/~janikow/publications/1992/GAforOpt/text.pdf
    a=1.0
    b=1.0
    q=1.0
    s=1.0
    r=1.0
    x0=100

    N=100

    X=SX.sym("X",N+1)
    U=SX.sym("U",N)

    V = vertcat(*[X,U])

    cost = 0
    for i in range(N):
      cost = cost + s*X[i]**2+r*U[i]**2
    cost = cost + q*X[N]**2

    nlp = {'x':V, 'f':cost, 'g':vertcat(*[X[0]-x0,X[1:,0]-(a*X[:N,0]+b*U)])}
    opts = {}
    opts["ipopt.tol"] = 1e-5
    opts["ipopt.hessian_approximation"] = "limited-memory"
    opts["ipopt.max_iter"] = 100
    opts["ipopt.print_level"] = 0
    solver = nlpsol("solver", "ipopt", nlp, opts)
    solver_in = {}
    solver_in["lbx"]=[-1000 for i in range(V.nnz())]
    solver_in["ubx"]=[1000 for i in range(V.nnz())]
    solver_in["lbg"]=[0 for i in range(N+1)]
    solver_in["ubg"]=[0 for i in range(N+1)]
    solver_out = solver(**solver_in)
    ocp_sol=solver_out["f"][0]
    # solve the ricatti equation exactly
    K = q+0.0
    for i in range(N):
      K = s+r*a**2*K/(r+b**2*K)
    exact_sol=K * x0**2
    self.assertAlmostEqual(ocp_sol,exact_sol,10,"Linear-quadratic problem solution using IPOPT")

  @requires_nlpsol("ipopt")
  def test_singleshooting(self):
    self.message("Single shooting")
    p0 = 0.2
    y0= 1
    yc0=dy0=0
    te=0.4

    t=SX.sym("t")
    q=SX.sym("y",2,1)
    p=SX.sym("p",1,1)
    # y
    # y'
    dae={'x':q, 'p':p, 't':t, 'ode':vertcat(*[q[1],p[0]+q[1]**2 ])}
    opts = {}
    opts["reltol"] = 1e-15
    opts["abstol"] = 1e-15
    opts["verbose"] = False
    opts["steps_per_checkpoint"] = 10000
    integrator = casadi.integrator("integrator", "cvodes", dae, 0, te, opts)

    var = MX.sym("var",2,1)
    par = MX.sym("par",1,1)
    parMX= par

    q0   = vertcat(*[var[0],par])
    par  = var[1]
    qend = integrator(x0=q0, p=par)["xf"]

    parc = MX(0)

    f = Function('f', [var,parMX],[qend[0]])
    nlp = {'x':var, 'f':-f(var,parc)}
    opts = {}
    opts["ipopt.tol"] = 1e-12
    opts["ipopt.hessian_approximation"] = "limited-memory"
    opts["ipopt.max_iter"] = 10
    opts["ipopt.derivative_test"] = "first-order"
    opts["ipopt.print_level"] = 0
    solver = nlpsol("solver", "ipopt", nlp, opts)
    solver_in = {}
    solver_in["lbx"]=[-1, -1]
    solver_in["ubx"]=[1, 0.2]
    solver_out = solver(**solver_in)
    print(solver_out["x"])
    self.assertAlmostEqual(solver_out["x"][0],1,7,"X_opt")
    self.assertAlmostEqual(solver_out["x"][1],0.2,7,"X_opt")
    self.assertAlmostEqual(fmax(solver_out["lam_x"],0)[0],1,8,"Cost should be linear in y0")
    self.assertAlmostEqual(fmax(solver_out["lam_x"],0)[1],(sqrt(p0)*(te*yc0**2-yc0+p0*te)*tan(arctan(yc0/sqrt(p0))+sqrt(p0)*te)+yc0**2)/(2*p0*yc0**2+2*p0**2),8,"Cost should be linear in y0")
    self.assertAlmostEqual(-solver_out["f"][0],(2*y0-log(yc0**2/p0+1))/2-log(cos(arctan(yc0/sqrt(p0))+sqrt(p0)*te)),7,"Cost")
    self.assertAlmostEqual(fmax(-solver_out["lam_x"],0)[0],0,8,"Constraint is supposed to be unactive")
    self.assertAlmostEqual(fmax(-solver_out["lam_x"],0)[1],0,8,"Constraint is supposed to be unactive")

  @requires_nlpsol("ipopt")
  def test_singleshooting2(self):
    self.message("Single shooting 2")
    p0 = 0.2
    y0= 0.2
    yc0=dy0=0.1
    te=0.4

    t=SX.sym("t")
    q=SX.sym("y",2,1)
    p=SX.sym("p",1,1)
    # y
    # y'
    dae={'x':q, 'p':p, 't':t, 'ode':vertcat(*[q[1],p[0]+q[1]**2 ])}
    opts = {}
    opts["reltol"] = 1e-15
    opts["abstol"] = 1e-15
    opts["verbose"] = False
    opts["steps_per_checkpoint"] = 10000
    integrator = casadi.integrator("integrator", "cvodes", dae, 0, te, opts)

    var = MX.sym("var",2,1)
    par = MX.sym("par",1,1)

    q0   = vertcat(*[var[0],par])
    parl  = var[1]
    qend = integrator(x0=q0,p=parl)["xf"]

    parc = MX(dy0)

    f = Function('f', [var,par],[qend[0]])
    nlp = {'x':var, 'f':-f(var,parc), 'g':var[0]-var[1]}
    opts = {}
    opts["ipopt.tol"] = 1e-12
    opts["ipopt.hessian_approximation"] = "limited-memory"
    opts["ipopt.max_iter"] = 10
    opts["ipopt.derivative_test"] = "first-order"
    #opts["ipopt.print_level"] = 0
    solver = nlpsol("solver", "ipopt", nlp, opts)
    solver_in = {}
    solver_in["lbx"]=[-1, -1]
    solver_in["ubx"]=[1, 0.2]
    solver_in["lbg"]=[-1]
    solver_in["ubg"]=[0]
    solver_out = solver(**solver_in)

    self.assertAlmostEqual(solver_out["x"][0],0.2,6,"X_opt")
    self.assertAlmostEqual(solver_out["x"][1],0.2,6,"X_opt")

    self.assertAlmostEqual(fmax(solver_out["lam_x"],0)[0],0,8,"Constraint is supposed to be unactive")
    dfdp0 = (sqrt(p0)*(te*yc0**2-yc0+p0*te)*tan(arctan(yc0/sqrt(p0))+sqrt(p0)*te)+yc0**2)/(2*p0*yc0**2+2*p0**2)
    self.assertAlmostEqual(fmax(solver_out["lam_x"],0)[1],1+dfdp0,8)
    self.assertAlmostEqual(solver_out["lam_g"][0],1,8)
    self.assertAlmostEqual(-solver_out["f"][0],(2*y0-log(yc0**2/p0+1))/2-log(cos(arctan(yc0/sqrt(p0))+sqrt(p0)*te)),7,"Cost")
    self.assertAlmostEqual(fmax(-solver_out["lam_x"],0)[0],0,8,"Constraint is supposed to be unactive")
    self.assertAlmostEqual(fmax(-solver_out["lam_x"],0)[1],0,8,"Constraint is supposed to be unactive")

  @requires_nlpsol("fatrop")
  def test_fatrop(self):
  
  
    flags = []
    if os.name != 'nt':
      flags = ["-Wno-strict-prototypes"]
  
    def test_problems():
    
        for i in range(2):

            T = 10. # Time horizon
            N = 10 # number of control intervals

            # Declare model variables
            x1 = MX.sym('x1')
            x2 = MX.sym('x2')
            x = vertcat(x1, x2)
            u = MX.sym('u')
            p = MX.sym('p')

            # Model equations
            xdot = vertcat((1-x2**2)*x1 - x2 + u+p, x1)

            F = integrator("F","rk",{"x":x,"p":p,"u":u,"ode":xdot}, 0, 1, {"simplify":True,"number_of_finite_elements":1})

            # Start with an empty NLP
            w=[]
            w0 = []
            lbw = []
            ubw = []
            J = 0
            g=[]
            lbg = []
            ubg = []
            equality = []

            # "Lift" initial conditions
            Xk = MX.sym('X0', 2)
            w += [Xk]
            lbw += [0, 1]
            ubw += [0, 1]
            w0 += [0.1, 0.2]

            # Formulate the NLP
            for k in range(N):
                # New NLP variable for the control
                Uk = MX.sym('U_' + str(k))
                w   += [Uk]
                lbw += [-1]
                ubw += [1]
                w0  += [0.3]

                # Integrate till the end of the interval
                Fk = F(x0=Xk, u=Uk, p=p)
                Xk_end = Fk['xf']
                J=J+sumsqr(Xk)+sumsqr(Uk)



                # New NLP variable for state at end of interval
                Xk_next = MX.sym('X_' + str(k+1), 2)
                w   += [Xk_next]
                lbw += [-0.25 if i==0 else -inf, -inf]
                ubw += [  inf,  inf]
                w0  += [0.1, 0.2]
                    
                # Add equality constraint
                g   += [Xk_next-Xk_end]
                lbg += [0, 0]
                ubg += [0, 0]
                equality+= [True,True]

                if i>=1:
                    g   += [sin(Xk[0])]
                    lbg += [-0.25]
                    ubg += [inf]
                    equality+= [False]
                    
                Xk = Xk_next
            if i>=2:
                    
                # "Lift" initial conditions
                Xk = MX.sym('X0', 2)
                w += [Xk]
                lbw += [-inf, -inf]
                ubw += [inf, inf]
                w0 += [0.1, 0.2]
                
                
                # Add equality constraint
                g   += [Xk_next-Xk]
                lbg += [0, 0]
                ubg += [0, 0]
                equality+= [True,True]

                # Formulate the NLP
                for k in range(N):
                    # New NLP variable for the control
                    Uk = MX.sym('U_' + str(k))
                    w   += [Uk]
                    lbw += [-0.1]
                    ubw += [0.1]
                    w0  += [0.3]

                    # Integrate till the end of the interval
                    Fk = F(x0=Xk, u=Uk, p=p)
                    Xk_end = Fk['xf']
                    J=J+3*sumsqr(Xk)+sumsqr(Uk)

                    # New NLP variable for state at end of interval
                    Xk_next = MX.sym('X_' + str(k+1), 2)
                    w   += [Xk_next]
                    lbw += [-inf, -inf]
                    ubw += [  inf,  inf]
                    w0  += [0.1, 0.2]
                        
                    # Add equality constraint
                    g   += [Xk_next-Xk_end]
                    lbg += [0, 0]
                    ubg += [0, 0]
                    equality+= [True,True]

                    Xk = Xk_next
                    
            if i>=3:
                # Declare model variables
                x = MX.sym('x',3)
                u = MX.sym('u',2)
                
                A = DM([[1,0,0.3],[0,1,0.7],[0.2,0,1]])
                B = DM([[1,0],[0,1],[0.5,0.5]])
                D = DM([[0.2,0.3],[0.8,0.7],[0.1,1]])

                F = Function("F",[x,u],[mtimes(A,x)+mtimes(B,u)])

                    
                # "Lift" initial conditions
                Xk = MX.sym('X0', 3)
                w += [Xk]
                lbw += [-inf, -inf, -inf]
                ubw += [inf, inf, inf]
                w0 += [0.1, 0.2, 0.3]
                
                
                # Add equality constraint
                g   += [mtimes(D,Xk_next)-Xk]
                lbg += [0, 0, 0]
                ubg += [0, 0, 0]
                equality+= [True,True, True]

                # Formulate the NLP
                for k in range(N):
                    # New NLP variable for the control
                    Uk = MX.sym('U_' + str(k),2)
                    w   += [Uk]
                    lbw += [-1,-1]
                    ubw += [1,1]
                    w0  += [0.3,0.3]

                    # Integrate till the end of the interval
                    Xk_end = F(Xk, Uk)
                    J=J+sumsqr(Xk)+sumsqr(Uk)

                    # New NLP variable for state at end of interval
                    Xk_next = MX.sym('X_' + str(k+1), 3)
                    w   += [Xk_next]
                    lbw += [-inf, -inf, -inf]
                    ubw += [  inf,  inf, inf]
                    w0  += [0.1, 0.2, 0.3]
                        
                    # Add equality constraint
                    g   += [Xk_next-Xk_end]
                    lbg += [0, 0, 0]
                    ubg += [0, 0, 0]
                    equality+= [True,True, True]

                    Xk = Xk_next
 
                        
                
            # Create an NLP solver
            yield {'f': J, 'x': vertcat(*w), 'g': vertcat(*g), 'p': p}, dict(x0=w0, lbx=lbw, ubx=ubw, lbg=lbg, ubg=ubg, p=0),equality
        
        for i in range(1):
            # Multi-stage with varying number of inequalities

            T = 10. # Time horizon
            N = 10 # number of control intervals

            # Declare model variables
            x1 = MX.sym('x1')
            x2 = MX.sym('x2')
            x = vertcat(x1, x2)
            u = MX.sym('u')
            p = MX.sym('p')

            # Model equations
            xdot = vertcat((1-x2**2)*x1 - x2 + u+p, x1)

            F = integrator("F","rk",{"x":x,"p":p,"u":u,"ode":xdot}, 0, 1, {"simplify":True,"number_of_finite_elements":1})

            # Start with an empty NLP
            w=[]
            w0 = []
            lbw = []
            ubw = []
            J = 0
            g=[]
            lbg = []
            ubg = []
            equality = []

            # "Lift" initial conditions
            Xk = MX.sym('X0', 2)
            w += [Xk]
            lbw += [0, 1]
            ubw += [0, 1]
            w0 += [0.1, 0.2]

            # Formulate the NLP
            for k in range(N):
                # New NLP variable for the control
                Uk = MX.sym('U1_' + str(k))
                w   += [Uk]
                lbw += [-1]
                ubw += [1]
                w0  += [0.3]

                # Integrate till the end of the interval
                Fk = F(x0=Xk, u=Uk, p=p)
                Xk_end = Fk['xf']
                J=J+sumsqr(Xk)+sumsqr(Uk)

                # New NLP variable for state at end of interval
                Xk_next = MX.sym('X1_' + str(k+1), 2)
                w   += [Xk_next]
                lbw += [-0.25, -inf]
                ubw += [  inf,  inf]
                w0  += [0.1, 0.2]
                    
                # Add equality constraint
                g   += [Xk_next-Xk_end]
                lbg += [0, 0]
                ubg += [0, 0]
                equality += [True, True]

                Xk = Xk_next

            # New NLP variable for the control
            Uk = MX.sym('U1')
            w   += [Uk]
            lbw += [-1]
            ubw += [1]
            w0  += [0.3]

            # Integrate till the end of the interval
            Fk = F(x0=Xk, u=Uk, p=p)
            Xk_end = Fk['xf']
            J=J+Xk[0]**2


            # New NLP variable for state at end of interval
            Xk = MX.sym('X1', 2)
            w   += [Xk]
            lbw += [-inf, -inf]
            ubw += [  inf,  inf]
            w0  += [0.1, 0.2]
                
            # Add equality constraint
            g   += [Xk-Xk_end]
            lbg += [0, 0]
            ubg += [0, 0]
            equality += [True, True]


            # "Lift" initial conditions
            #Xk = MX.sym('X0', 2)
            #w += [Xk]
            #lbw += [-inf, -inf]
            #ubw += [inf, inf]
            #w0 += [0.1, 0.2]


            # Add equality constraint
            #g   += [Xk_next-Xk]
            #lbg += [0, 0]
            #ubg += [0, 0]

            A = DM([[1,0.1],[0.2,1.1]])
            B = DM([[0.2],[0.7]])

            F = Function("F",[x,u],[mtimes(A,x)+mtimes(B,u)])

            # Formulate the NLP
            for k in range(N):
                # New NLP variable for the control
                Uk = MX.sym('U2_' + str(k))
                w   += [Uk]
                lbw += [-inf]
                ubw += [inf]
                w0  += [0.3]


                # Integrate till the end of the interval
                Xk_end = F(Xk, Uk)
                J=J+3*sumsqr(Xk)+sumsqr(Uk)

                # New NLP variable for state at end of interval
                Xk_next = MX.sym('X2_' + str(k+1), 2)
                w   += [Xk_next]
                lbw += [-inf, -inf]
                ubw += [  inf,  inf]
                w0  += [0.1, 0.2]
                    
                # Add equality constraint
                g   += [Xk_next-Xk_end]
                lbg += [0, 0]
                ubg += [0, 0]
                equality += [True, True]

                g   += [2*Uk]
                lbg += [-0.1]
                ubg += [0.1]
                equality += [False]


                Xk = Xk_next
        
        
            # Create an NLP solver
            yield {'f': J, 'x': vertcat(*w), 'g': vertcat(*g), 'p': p}, dict(x0=w0, lbx=lbw, ubx=ubw, lbg=lbg, ubg=ubg, p=0), equality
        
        for i in range(1):
            # Equality constraints

            T = 10. # Time horizon
            N = 10 # number of control intervals

            # Declare model variables
            x1 = MX.sym('x1')
            x2 = MX.sym('x2')
            x = vertcat(x1, x2)
            u1 = MX.sym('u1')
            u2 = MX.sym('u2')
            u = vertcat(u1, u2)
            p = MX.sym('p')

            # Model equations
            xdot = vertcat((1-x2**2)*x1 - x2 + u1+p, x1+u2)

            F = integrator("F","rk",{"x":x,"p":p,"u":u,"ode":xdot}, 0, 1, {"simplify":True,"number_of_finite_elements":1})

            # Start with an empty NLP
            w=[]
            w0 = []
            lbw = []
            ubw = []
            J = 0
            g=[]
            lbg = []
            ubg = []
            equality = []

            # "Lift" initial conditions
            Xk = MX.sym('X0', 2)
            w += [Xk]
            lbw += [0, 1]
            ubw += [0, 1]
            w0 += [0.1, 0.2]

            # Formulate the NLP
            for k in range(N):
                # New NLP variable for the control
                Uk = MX.sym('U_' + str(k),2)
                w   += [Uk]
                lbw += [-1,0.1]
                ubw += [1,0.1]
                w0  += [0.3,0]

                # Integrate till the end of the interval
                Fk = F(x0=Xk, u=Uk, p=p)
                Xk_end = Fk['xf']
                J=J+sumsqr(Xk)+sumsqr(Uk)



                # New NLP variable for state at end of interval
                Xk_next = MX.sym('X_' + str(k+1), 2)
                w   += [Xk_next]
                lbw += [-0.25 if i==0 else -inf, -inf]
                ubw += [  inf,  inf]
                w0  += [0.1, 0.2]
                    
                # Add equality constraint
                g   += [Xk_next-Xk_end]
                lbg += [0, 0]
                ubg += [0, 0]
                equality += [True,True]

                Xk = Xk_next
            # Create an NLP solver
            yield {'f': J, 'x': vertcat(*w), 'g': vertcat(*g), 'p': p}, dict(x0=w0, lbx=lbw, ubx=ubw, lbg=lbg, ubg=ubg, p=0), equality
            

        T = 10. # Time horizon
        N = 10 # number of control intervals

        # Declare model variables
        x1 = MX.sym('x1')
        x2 = MX.sym('x2')
        x = vertcat(x1, x2)
        u = MX.sym('u')
        p = MX.sym('p')

        # Model equations
        xdot = vertcat((1-x2**2)*x1 - x2 + u+p, x1)

        F = integrator("F","rk",{"x":x,"p":p,"u":u,"ode":xdot}, 0, 1, {"simplify":True,"number_of_finite_elements":1})

        # Start with an empty NLP
        w=[]
        w0 = []
        lbw = []
        ubw = []
        J = 0
        g=[]
        lbg = []
        ubg = []
        equality = []

        # "Lift" initial conditions
        Xk = MX.sym('X0', 2)
        w += [Xk]
        lbw += [0, 1]
        ubw += [0, 1]
        w0 += [0.1, 0.2]

        # Formulate the NLP
        for k in range(N):
            # New NLP variable for the control
            Uk = MX.sym('U1_' + str(k))
            w   += [Uk]
            lbw += [-1]
            ubw += [1]
            w0  += [0.3]

            # Integrate till the end of the interval
            Fk = F(x0=Xk, u=Uk, p=p)
            Xk_end = Fk['xf']
            J=J+sumsqr(Xk)+sumsqr(Uk)

            # New NLP variable for state at end of interval
            Xk_next = MX.sym('X1_' + str(k+1), 2)
            w   += [Xk_next]
            lbw += [-0.25, -inf]
            ubw += [  inf,  inf]
            w0  += [0.1, 0.2]
                
            # Add equality constraint
            g   += [Xk_next-Xk_end]
            lbg += [0, 0]
            ubg += [0, 0]
            equality += [True,True]

            Xk = Xk_next

        J=J+Xk[0]**2

        A = DM([[1,0,0.3],[0,1,0.7],[0.2,0,1]])
        B = DM([[1,0],[0,1],[0.5,0.5]])
        D = DM([[0.2,0.3],[0.8,0.7],[0.1,1]])

        # New NLP variable for state at end of interval
        Xk = MX.sym('X1', 3)
        w   += [Xk]
        lbw += [-inf, -inf, -inf]
        ubw += [  inf,  inf, inf]
        w0  += [0.7, 0.8, 0.9]
            
        # Add equality constraint
        g   += [Xk-mtimes(D,Xk_next)]
        lbg += [0, 0, 0]
        ubg += [0, 0, 0]
        equality += [True,True,True]

        u = MX.sym("u",2)
        x = MX.sym("x",3)
        F = Function("F",[x,u],[mtimes(A,x)+mtimes(B,u)])

        # Formulate the NLP
        for k in range(N):
            # New NLP variable for the control
            Uk = MX.sym('U2_' + str(k),2)
            w   += [Uk]
            lbw += [-0.1,-0.1]
            ubw += [0.1,0.1]
            w0  += [1.3,1.2]


            # Integrate till the end of the interval
            Xk_end = F(Xk, Uk)
            J=J+3*sumsqr(Xk)+sumsqr(Uk)

            # New NLP variable for state at end of interval
            Xk_next = MX.sym('X2_' + str(k+1), 3)
            w   += [Xk_next]
            lbw += [-inf, -inf, -inf]
            ubw += [  inf,  inf, inf]
            w0  += [0.7, 0.8, 0.9]
                
            # Add equality constraint
            g   += [Xk_next-Xk_end]
            lbg += [0, 0, 0]
            ubg += [0, 0, 0]
            equality += [True,True,True]


            Xk = Xk_next
 
         # Create an NLP solver
        yield {'f': J, 'x': vertcat(*w), 'g': vertcat(*g), 'p': p}, dict(x0=w0, lbx=lbw, ubx=ubw, lbg=lbg, ubg=ubg, p=0), equality
        
    for i,(prob,args,equality) in enumerate(test_problems()):
    
        jacobian_sparsity(prob["g"],prob["x"]).spy()

        solutions = {}
        stats = {}
        for solver, solver_options in [("ipopt",{}),("fatrop",{"structure_detection":"auto","fatrop":{"accept_every_trial_step":False,"tol":1e-8,"max_iter":100},"equality":equality})]:
            f = nlpsol('solver', solver, prob, solver_options)
            #if solver=="fatrop" and i==2: raise Exception() 

            # Solve the NLP
            solutions[solver] = f(**args)
            stats[solver] = f.stats()
            
            if solver!="ipopt":
                self.check_codegen(f,args,std="c99",extralibs=["fatrop","blasfeo"],extra_options=flags,digits=codegen_check_digits)
                self.check_serialize(f,args)
        
        for k in solutions["ipopt"].keys():
            if k in ["x","f","g","lam_g","lam_x","lam_p"]:
                v_ref = solutions["ipopt"][k]
                v = solutions["fatrop"][k]
                
                self.checkarray(v,v_ref,failmessage=k,digits=6)
        assert(abs(stats["ipopt"]["iter_count"]-stats["fatrop"]["iter_count"])<=2)



  @requires_nlpsol("fatrop")
  def test_fatrop_sanitize(self):
  
  
    def test_problems():
    

            
            T = 10. # Time horizon
            N = 10 # number of control intervals

            # Declare model variables
            x1 = MX.sym('x1')
            x2 = MX.sym('x2')
            x = vertcat(x1, x2)
            u = MX.sym('u')
            p = MX.sym('p')

            # Model equations
            xdot = vertcat((1-x2**2)*x1 - x2 + u+p, x1)

            F = integrator("F","rk",{"x":x,"p":p,"u":u,"ode":xdot}, 0, 1, {"simplify":True,"number_of_finite_elements":1})

            # Start with an empty NLP
            w=[]
            w0 = []
            lbw = []
            ubw = []
            J = 0
            g=[]
            equality = []
            lbg = []
            ubg = []

            # "Lift" initial conditions
            Xk = MX.sym('X0', 2)
            w += [Xk]
            lbw += [0, 1]
            ubw += [0, 1]
            w0 += [0.1, 0.2]

            # Formulate the NLP
            for k in range(N):
                # New NLP variable for the control
                Uk = MX.sym('U_' + str(k))
                w   += [Uk]
                lbw += [-1]
                ubw += [1]
                w0  += [0.3]

                # Integrate till the end of the interval
                Fk = F(x0=Xk, u=Uk, p=p)
                Xk_end = Fk['xf']
                J=J+sumsqr(Xk)+sumsqr(Uk)



                # New NLP variable for state at end of interval
                Xk_next = MX.sym('X_' + str(k+1), 2)
                w   += [Xk_next]
                lbw += [-0.25, -inf]
                ubw += [  inf,  inf]
                w0  += [0.1, 0.2]
                    
                # Add equality constraint
                g   += [Xk_end-Xk_next]
                lbg += [0, 0]
                ubg += [0, 0]
                equality+= [True, True]

                    
                Xk = Xk_next
           
 
             # Create an NLP solver
            yield {'f': J, 'x': vertcat(*w), 'g': vertcat(*g), 'p': p}, dict(x0=w0, lbx=lbw, ubx=ubw, lbg=lbg, ubg=ubg, p=0),equality
        
    for i,(prob,args,equality) in enumerate(test_problems()):
    
        jacobian_sparsity(prob["g"],prob["x"]).spy()


        for solver, solver_options in [("fatrop",{"structure_detection": "auto", "verbose": True, "equality": equality})]:
            f = nlpsol('solver', solver, prob, solver_options)
            #if solver=="fatrop" and i==2: raise Exception() 

            # Solve the NLP
            with self.assertInAnyOutput("gap-closing"):
                f(**args)
       
  @requires_nlpsol("fatrop")
  def test_detect(self):

    for nx1 in [2]:
        for nx2 in [2,0]:
            for nu1 in [2,0]:
                for nu2 in [2,0]:
                    for ng1 in [2,0]:
                        for ng2 in [2,0]:
                            for ng3 in [2,0]:
                                print("test_detect",nx1,nx2,nu1,nu2,ng1,ng2,ng3)
                                self.fatrop_case(N=2,nx0=2,nu0=2,nx1=nx1,nu1=nu1,nx2=nx2,nu2=nu2,ng1=ng1,ng2=ng2,ng3=ng3)

  @requires_nlpsol("fatrop")
  def test_detect_adversarial(self):
    
    D2 = sparsify(blockcat([[1,0,0],[1,1,1]])).sparsity()
    self.fatrop_case(nu2=3,sp={"D2": D2})
    
    D2 = sparsify(blockcat([[1,0,0],[1,1,1]])).sparsity()
    C2 = sparsify(blockcat([[0,1],[0,0]])).sparsity()
    self.fatrop_case(nu2=3,sp={"D2": D2, "C2": C2})
    with self.assertInAnyOutput("gap-closing constraints must be like"):
        self.fatrop_case(nu2=3,sp={"D2": D2, "C2": C2},eq={'ng3'}) # Why is this not trig
    
    self.fatrop_case(nu0=0,nx0=1)
    
    self.fatrop_case(nx2=0)
    
    
    #with self.assertInAnyOutput("Gap-closing constraint must depend on a state"):
    self.fatrop_case(nu2=3,sp={"A1": Sparsity(2,2), "B1": Sparsity(2,2)})
    #with self.assertInAnyOutput("Gap-closing constraint must depend on a state"):
    self.fatrop_case(nu2=3,sp={"A1": Sparsity(2,2)})
        
    self.fatrop_case(nx2=1,ng1=0,nu1=0)
    
  @requires_nlpsol("fatrop")
  def test_bug(self):

    x = ca.MX.sym("x")

    for structure_detection in ["none","auto"]:

        opts = {"expand": True, "structure_detection": structure_detection,"equality":[True]}
        
        solver = ca.nlpsol("solver","fatrop",{"x":x,"g":x-1},opts)
        self.assertAlmostEqual(solver(lbg=0,ubg=0)["x"],1,5)

        solver = ca.nlpsol("solver","fatrop",{"x":x,"g":x},opts)
        self.assertAlmostEqual(solver(lbg=1,ubg=1)["x"],1,5)

        solver = ca.nlpsol("solver","fatrop",{"x":x,"g":x-2},opts)
        self.assertAlmostEqual(solver(lbg=3,ubg=3)["x"],5,5)
        
  
if __name__ == '__main__':
    unittest.main()



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

# For documentation about this examples, check http://docs.casadi.org/documents/mhe_spring_damper.pdf

from casadi import *
import numpy as NP
import matplotlib.pyplot as plt
import time
from casadi.tools import *
from scipy import linalg, matrix
plt.interactive(True)

NP.random.seed(0)

# Settings of the filter
N = 10 # Horizon length
dt = 0.05; # Time step

sigma_p = 0.005 # Standard deviation of the position measurements
sigma_w = 0.1 # Standard deviation for the process noise
R = DM(1/sigma_p**2) # resulting weighting matrix for the position measurements
Q = DM(1/sigma_w**2) # resulting weighting matrix for the process noise

Nsimulation = 1000 # Lenght of the simulation

# Parameters of the system
m = 1 # The weight of the mass
k = 1 # The spring constant
c = 0.5 # The damping of the system
# The state
states = struct_symSX(["x","dx"]) # Full state vector of the system: position x and velocity dx
Nstates = states.size # Number of states
# Set up some aliases
x,dx = states[...]

# The control input
controls = struct_symSX(["F"]) # Full control vector of the system: Input force F
Ncontrols = controls.size # Number of control inputs
# Set up some aliases
F, = controls[...]

# Disturbances
disturbances = struct_symSX(["w"]) # Process noise vector
Ndisturbances = disturbances.size # Number of disturbances
# Set up some aliases
w, = disturbances[...]

# Measurements
measurements = struct_symSX(["y"]) # Measurement vector
Nmeas = measurements.size # Number of measurements
# Set up some aliases
y, = measurements[...]

# Create Structure for the entire horizon

# Structure that will be degrees of freedom for the optimizer
shooting = struct_symSX([(entry("X",repeat=N,struct=states),entry("W",repeat=N-1,struct=disturbances))])
# Structure that will be fixed parameters for the optimizer
parameters = struct_symSX([(entry("U",repeat=N-1,struct=controls),entry("Y",repeat=N,struct=measurements),entry("S",shape=(Nstates,Nstates)),entry("x0",shape=(Nstates,1)))])
S = parameters["S"]
x0 = parameters["x0"]
# Define the ODE right hand side
rhs = struct_SX(states)
rhs["x"] = dx
rhs["dx"] = (-k*x-c*dx+F)/m+w

f = Function('f', [states,controls,disturbances],[rhs])

# Build an integrator for this system: Runge Kutta 4 integrator
k1 = f(states,controls,disturbances)
k2 = f(states+dt/2.0*k1,controls,disturbances)
k3 = f(states+dt/2.0*k2,controls,disturbances)
k4 = f(states+dt*k3,controls,disturbances)

states_1 = states+dt/6.0*(k1+2*k2+2*k3+k4)
phi = Function('phi', [states, controls, disturbances], [states_1], ['x', 'u', 'd'], ['x1'])
PHI = phi.factory('PHI', ['x', 'u', 'd'], ['jac:x1:x'])
# Define the measurement system
h = Function('h', [states], [x], ['x'], ['y']) # We have measurements of the position
H = h.factory('H', ['x'], ['jac:y:x'])
# Build the objective
obj = 0
# First the arrival cost
obj += mtimes([(shooting["X",0]-parameters["x0"]).T,S,(shooting["X",0]-parameters["x0"])])
#Next the cost for the measurement noise
for i in range(N):
  vm = h(shooting["X",i])-parameters["Y",i]
  obj += mtimes([vm.T,R,vm])
#And also the cost for the process noise
for i in range(N-1):
  obj += mtimes([shooting["W",i].T,Q,shooting["W",i]])

# Build the multiple shooting constraints
g = []
for i in range(N-1):
  g.append( shooting["X",i+1] - phi(shooting["X",i],parameters["U",i],shooting["W",i]) )

# Formulate the NLP
nlp = {'x':shooting, 'p':parameters, 'f':obj, 'g':vertcat(*g)}

# Make a simulation to create the data for the problem
simulated_X = DM.zeros(Nstates,Nsimulation)
simulated_X[:,0] = DM([1,0]) # Initial state
t = NP.linspace(0,(Nsimulation-1)*dt,Nsimulation) # Time grid
simulated_U = DM(cos(t[0:-1])).T # control input for the simulation
simulated_U[:,int(Nsimulation/2):] = 0.0
simulated_W = DM(sigma_w*NP.random.randn(Ndisturbances,Nsimulation-1)) # Process noise for the simulation
for i in range(Nsimulation-1):
  simulated_X[:,i+1] = phi(simulated_X[:,i], simulated_U[:,i], simulated_W[:,i])
#Create the measurements from these states
simulated_Y = DM.zeros(Nmeas,Nsimulation) # Holder for the measurements
for i in range(Nsimulation):
  simulated_Y[:,i] = h(simulated_X[:,i])
# Add noise the the position measurements
simulated_Y += sigma_p*NP.random.randn(simulated_Y.shape[0],simulated_Y.shape[1])

#The initial estimate and related covariance, which will be used for the arrival cost
sigma_x0 = 0.01
P = sigma_x0**2*DM.eye(Nstates)
x0 = simulated_X[:,0] + sigma_x0*NP.random.randn(Nstates,1)
# Create the solver
opts = {"ipopt.print_level":0, "print_time": False, 'ipopt.max_iter':100}
nlpsol = nlpsol("nlpsol", "ipopt", nlp, opts)

# Create a holder for the estimated states and disturbances
estimated_X= DM.zeros(Nstates,Nsimulation)
estimated_W = DM.zeros(Ndisturbances,Nsimulation-1)
# For the first instance we run the filter, we need to initialize it.
current_parameters = parameters(0)
current_parameters["U",lambda x: horzcat(*x)] = simulated_U[:,0:N-1]
current_parameters["Y",lambda x: horzcat(*x)] = simulated_Y[:,0:N]
current_parameters["S"] = linalg.inv(P) # Arrival cost is the inverse of the initial covariance
current_parameters["x0"] = x0
initialisation_state = shooting(0)
initialisation_state["X",lambda x: horzcat(*x)] = simulated_X[:,0:N]
res = nlpsol(p=current_parameters, x0=initialisation_state, lbg=0, ubg=0)

# Get the solution
solution = shooting(res["x"])
estimated_X[:,0:N] = solution["X",lambda x: horzcat(*x)]
estimated_W[:,0:N-1] = solution["W",lambda x: horzcat(*x)]

# Now make a loop for the rest of the simulation
for i in range(1,Nsimulation-N+1):

  # Update the arrival cost, using linearisations around the estimate of MHE at the beginning of the horizon (according to the 'Smoothed EKF Update'): first update the state and covariance with the measurement that will be deleted, and next propagate the state and covariance because of the shifting of the horizon
  print("step %d/%d (%s)" % (i, Nsimulation-N , nlpsol.stats()["return_status"]))
  H0 = H(solution["X",0])
  K = mtimes([P,H0.T,linalg.inv(mtimes([H0,P,H0.T])+R)])
  P = mtimes((DM.eye(Nstates)-mtimes(K,H0)),P)
  h0 = h(solution["X",0])
  x0 = x0 + mtimes(K, current_parameters["Y",0]-h0-mtimes(H0,x0-solution["X",0]))
  x0 = phi(x0, current_parameters["U",0], solution["W",0])
  F = PHI(solution["X",0], current_parameters["U",0], solution["W",0])
  P = mtimes([F,P,F.T]) + linalg.inv(Q)
  # Get the measurements and control inputs
  current_parameters["U",lambda x: horzcat(*x)] = simulated_U[:,i:i+N-1]
  current_parameters["Y",lambda x: horzcat(*x)] = simulated_Y[:,i:i+N]
  current_parameters["S"] = linalg.inv(P)
  current_parameters["x0"] = x0
  # Initialize the system with the shifted solution
  initialisation_state["W",lambda x: horzcat(*x),0:N-2] = estimated_W[:,i:i+N-2] # The shifted solution for the disturbances
  initialisation_state["W",N-2] = DM.zeros(Ndisturbances,1) # The last node for the disturbances is initialized with zeros
  initialisation_state["X",lambda x: horzcat(*x),0:N-1] = estimated_X[:,i:i+N-1] # The shifted solution for the state estimates
  # The last node for the state is initialized with a forward simulation
  phi0 = phi(initialisation_state["X",N-1], current_parameters["U",-1], initialisation_state["W",-1])
  initialisation_state["X",N-1] = phi0
  # And now initialize the solver and solve the problem
  res = nlpsol(p=current_parameters, x0=initialisation_state, lbg=0, ubg=0)
  solution = shooting(res["x"])

  # Now get the state estimate. Note that we are only interested in the last node of the horizon
  estimated_X[:,N-1+i] = solution["X",N-1]
  estimated_W[:,N-2+i] = solution["W",N-2]
# Plot the results
plt.figure(1)
plt.clf()
plt.plot(t,vec(estimated_X[0,:]),'b--')
plt.plot(t,vec(simulated_X[0,:]),'r--')
plt.title("Position")
plt.xlabel('Time')
plt.legend(['Estimated position','Real position'])
plt.grid()

plt.figure(2)
plt.clf()
plt.plot(t,vec(estimated_X[0,:]-simulated_X[0,:]),'b--')
plt.title("Position error")
plt.xlabel('Time')
plt.legend(['Error between estimated and real position'])
plt.grid()

plt.show()

error = estimated_X[0,:]-simulated_X[0,:]
print(mtimes(error,error.T))
assert(mtimes(error,error.T)<0.01)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

#
# NLPImplicitSolver
# =====================

from casadi import *
from numpy import *
from pylab import *

# We will investigate the working of rootfinder with the help of the parametrically exited Duffing equation.
#
# $\ddot{u}+\dot{u}-\epsilon (2 \mu \dot{u}+\alpha u^3+2 k u \cos(\Omega t))$ with $\Omega = 2 + \epsilon \sigma$. \\
#
# The first order solution is $u(t)=a \cos(\frac{1}{2} \Omega t-\frac{1}{2} \gamma)$ with the modulation equations: \\
# $\frac{da}{d\epsilon t} = - \left[ \mu a + \frac{1}{2} k a \sin \gamma \right]$ \\
# $a \frac{d\gamma}{d\epsilon t} = - \left[ -\sigma a + \frac{3}{4} \alpha a^3 + k a \cos \gamma \right]$ \\
#
# We seek the stationair solution to these modulation equations.

# Parameters

eps   = SX.sym("eps")
mu    = SX.sym("mu")
alpha = SX.sym("alpha")
k     = SX.sym("k")
sigma = SX.sym("sigma")
params = [eps,mu,alpha,k,sigma]

# Variables

a     = SX.sym("a")
gamma = SX.sym("gamma")

# Equations

res0 = mu*a+1.0/2*k*a*sin(gamma)
res1 = -sigma * a + 3.0/4*alpha*a**3+k*a*cos(gamma)

# Numerical values

sigma_ = 0.1
alpha_ = 0.1
k_     = 0.2
params_ = [0.1,0.1,alpha_,k_,sigma_]

# We create a NLPImplicitSolver instance

f=Function("f", [vertcat(a, gamma), vertcat(*params)], [vertcat(res0, res1)])
opts = {}
opts["nlpsol"] = "ipopt"
opts["nlpsol_options"] = {"ipopt.tol":1e-14}
s=rootfinder("s", "nlpsol", f, opts)

# Initialize [$a$,$\gamma$] with a guess and solve

x_ = s([1,-1], params_)
print("Solution = ", x_)

# Compare with the analytic solution:

x = [sqrt(4.0/3*sigma_/alpha_),-0.5*pi]
print("Reference solution = ", x)

# We show that the residual is indeed (close to) zero

residual = f(x_, params_)
print("residual = ", residual)

for i in range(1):
  assert(abs(x_[i]-x[i])<1e-6)

# Solver statistics

print(s.stats())



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
from casadi import *

"""
Solves the following optimal control problem (OCP) in differential-algebraic
equations (DAE)

minimize     integral_{t=0}^{10} x0^2 + x1^2 + u^2  dt
x0,x1,z,u

subject to   dot(x0) == z*x0-x1+u     \
             dot(x1) == x0             }  for 0 <= t <= 10
                   0 == x1^2 + z - 1  /
             x0(t=0) == 0
             x1(t=0) == 1
             x0(t=10) == 0
             x1(t=10) == 0
             -0.75 <= u <= 1  for 0 <= t <= 10

The method used is direct multiple shooting.

Joel Andersson, 2015
"""

# Declare variables
x0 = SX.sym('x0')
x1 = SX.sym('x1')
x = vertcat(x0, x1) # Differential states
z = SX.sym('z')       # Algebraic variable
u = SX.sym('u')       # Control

# Differential equation
f_x = vertcat(z*x0-x1+u, x0)

# Algebraic equation
f_z = x1**2 + z - 1

# Lagrange cost term (quadrature)
f_q = x0**2 + x1**2 + u**2

# Create an integrator
dae = {'x':x, 'z':z, 'p':u, 'ode':f_x, 'alg':f_z, 'quad':f_q}
# interval length 0.5s
I = integrator('I', 'idas', dae, 0, 0.5)

# Number of intervals
nk = 20

# Start with an empty NLP
w = []   # List of variables
lbw = [] # Lower bounds on w
ubw = [] # Upper bounds on w
G = []   # Constraints
J = 0    # Cost function

# Initial conditions
Xk = MX.sym('X0', 2)
w.append(Xk)
lbw += [ 0, 1 ]
ubw += [ 0, 1 ]

# Loop over all intervals
for k in range(nk):
  # Local control
  Uk = MX.sym('U'+str(k))
  w.append(Uk)
  lbw += [-0.75]
  ubw += [ 1.00]

  # Call integrator function
  Ik = I(x0=Xk, p=Uk)
  Xk = Ik['xf']
  J = J + Ik['qf'] # Sum quadratures

  # "Lift" the variable
  X_prev = Xk
  Xk = MX.sym('X'+str(k+1), 2)
  w.append(Xk)
  lbw += [-inf, -inf]
  ubw += [ inf,  inf]
  G.append(X_prev - Xk)

# Allocate an NLP solver
nlp = {'x':vertcat(*w), 'f':J, 'g':vertcat(*G)}
opts = {'ipopt.linear_solver':'ma27'}
solver = nlpsol('solver', 'ipopt', nlp, opts)

# Pass bounds, initial guess and solve NLP
sol = solver(lbx = lbw, # Lower variable bound
             ubx = ubw,  # Upper variable bound
             lbg = 0.0,  # Lower constraint bound
             ubg = 0.0,  # Upper constraint bound
             x0  = 0.0) # Initial guess

# Plot the results
import matplotlib.pyplot as plt
plt.figure(1)
plt.clf()
plt.plot(linspace(0., 10., nk+1), sol['x'][0::3],'--')
plt.plot(linspace(0., 10., nk+1), sol['x'][1::3],'-')
plt.plot(linspace(0., 10., nk), sol['x'][2::3],'-.')
plt.title('Van der Pol optimization - multiple shooting')
plt.xlabel('time')
plt.legend(['x0 trajectory','x1 trajectory','u trajectory'])
plt.grid()
plt.show()



# Python
from casadi import *

# Formulate the NLP
x=SX.sym('x')
y=SX.sym('y')
z=SX.sym('z')
f=x**2+100*z**2
g=z+(1-x)**2-y
P=dict(x=vertcat(x,y,z),\
       f=f,g=g)

# Create solver instance
F=nlpsol('F','ipopt',P)

# Solve the problem
r=F(x0=[2.5,3.0,0.75],\
    ubg=0, lbg=0)
print(r['x'])



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#

# n_nodes
# ======================

from casadi import *
from casadi.tools import *

# Let's build a trivial symbolic SX graph

x = SX.sym("x")
y = SX.sym("y")
z = x*y+2*y
print(n_nodes(z), " nodes in ", z)
dotdraw(z)

z += 4*z
print(n_nodes(z), " nodes in ", z)
dotdraw(z)

z *= z+1
print(n_nodes(z), " nodes in ", z)
dotdraw(z)





#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
from numpy import random, array, linalg, matrix, zeros, ones, ndarray, eye
import unittest
from types import *
from helpers import *
from copy import deepcopy

import sys

if sys.version_info >= (3, 0):
  TupleType = tuple


warnings.filterwarnings("ignore",category=DeprecationWarning)

scipy_available = True
try:
	from scipy.sparse import csr_matrix
	import scipy
	import scipy.linalg
except:
	scipy_available = False

dplesolvers = []

if has_dple("slicot"):
  dplesolvers.append(("slicot",{"linear_solver": "csparse"}))

def randstable(n,margin=0.8,minimal=0):
  r = margin
  A_ = tril(DM(numpy.random.random((n,n))))
  for i in range(n):
    eig = 0
    while abs(eig)<=minimal:
      eig = numpy.random.random()*r*2-r
    A_[i,i] = eig

  Q = scipy.linalg.orth(numpy.random.random((n,n)))
  return mtimes([Q,A_,Q.T])

class Controltests(casadiTestCase):

  @skip(not scipy_available)
  @memory_heavy()
  def test_dple_small(self):
    
    for Solver, options in dplesolvers:
      for K in ([3,4] if args.run_slow else [2,3]):
        for n in [2,3,4]:
          print (Solver, options)
          numpy.random.seed(1)
          print (n,K)
          A_ = [randstable(n) for i in range(K)]
          
          V_ = [mtimes(v,v.T) for v in [DM(numpy.random.random((n,n))) for i in range(K)]]
          V2_ = [mtimes(v,v.T) for v in [DM(numpy.random.random((n,n))) for i in range(K)]]
          S = kron(Sparsity.diag(K),Sparsity.dense(n,n))
          solver = dplesol("solver", Solver,{'a':S,'v':repmat(S,1,2)}, options)
          
          inputs = {"a":dcat(A_), "v": horzcat(dcat(V_),dcat(V2_))}
          
          As = MX.sym("A",S)
          Vs = MX.sym("V",S)
          
          def sigma(a):
            return a[1:] + [a[0]]
            
          def isigma(a):
            return [a[-1]] + a[:-1]
          
          Vss = hcat([(i+i.T)/2 for i in isigma(list(diagsplit(Vs,n))) ])
          
          
          AA = dcat([c.kron(i,i) for i in diagsplit(As,n)])

          A_total = DM.eye(n*n*K) - vertcat(*[AA[-n*n:,:],AA[:-n*n,:]])
          
          
          Pf = solve(A_total,vec(Vss),"csparse")
          P = Pf.reshape((n,K*n))
          P = dcat(horzsplit(P,n))
          
          refsol = Function("refsol", {"a": As,"v":Vs,"p":P},dple_in(),dple_out()).map("map","serial",2,["a"],[])

          self.checkfunction(solver,refsol,inputs=inputs,failmessage=str(Solver))
    
  @skip(not scipy_available)
  @memory_heavy()
  def test_dple_alt_small(self):
    
    for Solver, options in dplesolvers:
      for K in ([3,4] if args.run_slow else [2,3]):
        for n in [2,3,4]:
          print (Solver, options)
          numpy.random.seed(1)
          print (n,K)
          A_ = [randstable(n) for i in range(K)]          
          V_ = [mtimes(v,v.T) for v in [DM(numpy.random.random((n,n))) for i in range(K)]]
        
          inputs = {"a":hcat(A_), "v": hcat(V_)}
          
          As = MX.sym("A",n,n*K)
          Vs = MX.sym("V",n,n*K)
                    
          def sigma(a):
            return a[1:] + [a[0]]
            
          def isigma(a):
            return [a[-1]] + a[:-1]
          
          Vss = hcat([(i+i.T)/2 for i in isigma(list(horzsplit(Vs,n))) ])
          
          
          AA = dcat([c.kron(i,i) for i in horzsplit(As,n)])

          A_total = DM.eye(n*n*K) - vcat([AA[-n*n:,:],AA[:-n*n,:]])
          
          Pf = solve(A_total,vec(Vss),"csparse")
          P = Pf.reshape((n,K*n))

          solver = Function("solver", {"a": As,"v":Vs,"p":hcat(dplesol(horzsplit(As,n),horzsplit(Vs,n),Solver,options))},dple_in(),dple_out())          
          refsol = Function("refsol", {"a": As,"v":Vs,"p":P},dple_in(),dple_out())

          self.checkfunction(solver,refsol,inputs=inputs,failmessage=str(Solver))
    
if __name__ == '__main__':
    unittest.main()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# -*- coding: utf-8 -*-

from .graph import *
from .bounds import *
import os

import sys
if sys.version_info >= (3,0):
  from .structure3 import repeated, entry, struct_symSX, struct_symMX, struct_SX, struct_MX, struct_MX_mutable, nesteddict, index, indexf, struct, struct_load
else:
  from .structure import repeated, entry, struct_symSX, struct_symMX, struct_SX, struct_MX, struct_MX_mutable, nesteddict, index, indexf, struct, struct_load
from .in_out import nice_stdout, capture_stdout

def print_subclasses(myclass, depth=0):
  print(("  " * depth) + " - " + myclass.__name__)
  for s in myclass.__subclasses__():
    print_subclasses(s,depth=depth+1)

def loadAllCompiledPlugins():
  for k in CasadiMeta.plugins().split(";"):
    cls, name = k.split("::")
    print("Testing: ", cls, name)
    if "SKIP_" + name.upper() + "_TESTS" in os.environ:
        print("Skipping")
        continue
    if cls in ("XmlFile"):
      pass
    elif cls in ("Importer","Linsol"):
      getattr(casadi,cls).load_plugin(name)
    else:
      getattr(casadi,'load_'+cls.lower())(name)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

#
# Monodromy matrix
# =====================

from casadi import *
from numpy import *
from pylab import *

# We will investigate the monodromy matrix with the help of a simple 2-state system, as found in 1. Nayfeh AH, Balachandran B. Applied nonlinear dynamics. 1995. Available at: http://onlinelibrary.wiley.com/doi/10.1002/9783527617548.biblio/summary [Accessed June 16, 2011], page 52.
# $\dot{x_1} = x_2$
# $\dot{x_2} = -(-w_0^2 x_1 + a_3 x_1^3 + a_5 x_1^5) - (2 mu_1 x_2 + mu_3 x_2^3) + f$.

x  = SX.sym("x",2)
x1 = x[0]
x2 = x[1]

w0 = SX.sym("w0")
a3 = SX.sym("a3")
a5 = SX.sym("a5")
mu1 = SX.sym("mu1")
mu3 = SX.sym("mu3")
ff = SX.sym("f")

tf = 40

params = vertcat(w0,a3,a5,mu1,mu3,ff)
rhs    = vertcat(x2,(-(-w0**2 *x1 + a3*x1**3 + a5*x1**5) - (2 *mu1 *x2 + mu3 * x2**3))/100+ff)

dae={'x':x, 'p':params, 'ode':rhs}

# t = SX.sym("t")
# cf=SX.fun("cf", controldaeIn(t=t, x=x, p=vertcat(w0,a3,a5,mu1,mu3), u=ff),[rhs])

# opts = {}
# opts["tf"] = tf
# opts["reltol"] = 1e-10
# opts["abstol"] = 1e-10
# opts["fsens_err_con"] = True
# integrator = integrator("integrator", "cvodes", dae, opts)

# N = 500

# # Let's get acquainted with the system by drawing a phase portrait
# ts = linspace(0,tf,N)

# sim = Simulator("sim", integrator,ts)

# w0_ = 5.278
# params_ = [ w0_, -1.402*w0_**2,  0.271*w0_**2,0,0,0 ]

# sim.setInput(params_,"p")

# x2_0 = 0
# figure(1)
# for x1_0 in [-3.5,-3.1,-3,-2,-1,0]:
#   sim.setInput([x1_0,x2_0],"x0")
#   sim.evaluate()
#   plot(sim.getOutput()[0,:],sim.getOutput()[1,:],'k')

# title('phase portrait for mu_1 = 0, mu_2 = 0')
# xlabel('x_1')
# ylabel('x_2')

# show()

# x0 = DM([-3.1,0])

# # Monodromy matrix at tf - Jacobian of integrator
# # ===============================================
# # First argument is input index, second argument is output index
# jac = integrator.jacobian("x0","xf")

# jac.setInput(x0,"x0")
# jac.setInput(params_,"p")
# jac.evaluate()

# Ji = jac.getOutput()

# print Ji
